#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : lifespans
# @Time         : 2024/12/13 10:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from contextlib import asynccontextmanager

from meutils.pipe import *
from meutils.config_utils.manager import ConfigManager


class Resources(BaseModel):
    config_manager: Optional[ConfigManager] = None

    class Config:
        # 允许额外字段，增加灵活性
        extra = 'allow'
        arbitrary_types_allowed = True


# def fake_answer_to_everything_ml_model(x: float):
#     return x * 42
#
#
# ml_models = {}
#
#
# @asynccontextmanager
# async def lifespan(app):
#     # Load the ML model
#     ml_models["answer_to_everything"] = fake_answer_to_everything_ml_model
#     yield
#     # Clean up the ML models and release the resources
#     ml_models.clear()


resources = Resources()


@asynccontextmanager
async def nacos_lifespan(app):
    import gc

    logger.debug("Init lifespans")

    config_manager = ConfigManager("test", "testdata")

    resources.config_manager = config_manager

    yield

    # release the resources
    logger.debug("Release the resources")
    del config_manager
    gc.collect()


if __name__ == '__main__':
    from fastapi import FastAPI

    app = FastAPI(lifespan=lifespan)

    app.state.config_manager = ConfigManager("test", "testdata")


    @app.get("/predict")
    async def predict(x: float):
        result = ml_models["answer_to_everything"](x)
        return {"result": result}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : 国产分组
# @Time         : 2024/11/26 16:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.schemas.openai_types import ChatCompletionRequest

from uptime_kuma_api import UptimeKumaApi, MonitorType

PAGE_NAME = "国产模型监控"
API_KEY = "sk-9MtwOUKXaAas1i0r3KNnsxBNKIrtQ07fY4hTbKso0ud6TY8j"
with UptimeKumaApi('https://status.chatfire.cn/') as api:
    _ = api.login('chatfire', 'chatfirechatfire!')

    # api.edit_monitor
    # print(api.get_monitors())
    #
    # api.delete_monitor(21)


    model = "doubao-pro-128k"
    model = "glm-4"

    result = api.add_monitor(
        type=MonitorType.HTTP,
        interval=5 * 60,  # 检测频率

        description=PAGE_NAME,
        name=model,

        method="POST",
        url="https://api.chatfire.cn/v1/chat/completions",
        headers={"Authorization": f"Bearer {API_KEY}"},
        body={
            "model": model,
            "messages": [{"role": "user", "content": "hi"}],
            "max_tokens": 1
        },
    )

    # api.add_status_page(title=PAGE_NAME, slug=PAGE_NAME)
    # print(api.get_status_page('guochan'))

    api.save_status_page(
        slug="guochan",

        id=PAGE_NAME,
        title=PAGE_NAME,

        publicGroupList=[
            {
                'name': PAGE_NAME,
                'weight': 1,
                'monitorList': [
                    {
                        "id": result.get('monitorID')
                    }
                ]
            }
        ]
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : video
# @Time         : 2024/7/16 10:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.schemas.runwayml_types import BASE_URL, RunwayRequest, EXAMPLES
from meutils.schemas.task_types import Task

from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=cx0SGO"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


# @retrying(max_retries=3, predicate=lambda x: not x)
async def get_access_token(token: Optional[str] = None):  # 暂时好像用不到待定
    token = token or await get_next_token_for_polling(FEISHU_URL)
    headers = {
        'Authorization': f'Bearer {token}',
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post('/short_jwt')
        if response.is_success:
            return response.json().get('token')


@retrying(max_retries=6, predicate=lambda r: not r)
async def create_task(request: Union[RunwayRequest, dict], token: Optional[str] = None) -> object:
    token = token or await get_next_token_for_polling(FEISHU_URL)

    headers = {
        'Authorization': f'Bearer {token}',
        'Content-type': 'application/json'
    }

    payload = request if isinstance(request, dict) else request.model_dump(exclude_none=True)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.post("/tasks", json=payload)

        # logger.debug(response.text)

        if response.is_success:
            data = response.json()
            send_message(bjson(data))

            try:
                tass_id = data.get("task").get("id")
                return Task(id=tass_id, data=data, system_fingerprint=token)

            except Exception as e:
                logger.error(e)
                send_message(f"未知错误：{e}")
        else:
            logger.debug(response.status_code)
            # 触发重试
            if (
                    any(response.status_code == code for code in {401, 429})
                    # or any(i in response.text for i in {"enough credits"})
            ):
                return
            else:
                send_message(f"{response.text}\n\n{token}")

                return Task(status=0, data=response.text)


@retrying(predicate=lambda r: not r)  # 触发重试
async def get_task(task_id: str, token: str):  # 1ee02f7b-42df-4926-b034-f56e4c4e2d31
    task_id = isinstance(task_id, str) and task_id.split("-", 1)[-1]

    headers = {
        'Authorization': f'Bearer {token}',
        'Content-type': 'application/json'
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.get(f"/tasks/{task_id}")

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.is_success:
            return response.json()
        else:
            return {"task": response.json()}  # 400 {"error":"id: The id specified in this URL must be a UUID"}


# 排队

# https://api.runwayml.com/v1/assets_pending?asTeamId=16871062&privateInTeam=true
# {
#     "pendingAssets": [
#         {
#             "id": "b218fed7-cd5f-4f8e-afc8-ac2b6365c9f0",
#             "createdAt": "2024-07-30T02:50:12.084Z",
#             "updatedAt": "2024-07-30T02:51:02.062Z",
#             "name": "Gen-2 8s, 227015205, M 5",
#             "previewUrls": [],
#             "mediaType": "video",
#             "mediaSubtype": null,
#             "isUserUpload": false,
#             "username": "niyudintg",
#             "progressRatio": "0.33",
#             "progressText": null
#         },
#         {
#             "id": "f7027b60-3fb8-4803-a830-44435deace81",
#             "createdAt": "2024-07-30T02:49:57.061Z",
#             "updatedAt": "2024-07-30T02:51:05.264Z",
#             "name": "Gen-2 8s, 3227464482, M 5",
#             "previewUrls": [],
#             "mediaType": "video",
#             "mediaSubtype": null,
#             "isUserUpload": false,
#             "username": "niyudintg",
#             "progressRatio": "0.41",
#             "progressText": null
#         }
#     ]
# }
if __name__ == '__main__':
    pass
    # print(arun(get_access_token()))
    print(arun(create_task(EXAMPLES[1])))
    # print(arun(create_task(RunwayRequest(**text2video_payload))))
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MTcyNDk1NTMsImVtYWlsIjoicGtmZ2JtbDc2NzBAb3V0bG9vay5jb20iLCJleHAiOjE3MjM2ODc5MTkuOTA5LCJpYXQiOjE3MjEwOTU5MTkuOTA5LCJzc28iOmZhbHNlfQ.CZLGYU0FXraU5zT-GY1CpbfQMOi9Srdo2QOXzob38PU"
    # task_id = "1ee02f7b-42df-4926-b034-f56e4c4e2d31"
    # task_id = "b7658f8a-6205-48ad-8655-4ad9b1ff856d"
    # print(arun(get_task(task_id, token)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : asr
# @Time         : 2023/5/17 13:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : pip install meutils openai-whisper pysrt opencc opencc-python-reimplemented -U
# export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/opt/miniconda/lib/python3.10/site-packages/nvidia/cudnn/lib

import pysrt
from datetime import timedelta

from meutils.pipe import *
from meutils.str_utils import chinese_convert

from faster_whisper import WhisperModel
from faster_whisper.transcribe import Segment

from whisper.normalizers import BasicTextNormalizer, EnglishTextNormalizer

class ASR(object):

    def __init__(self, model_size_or_path='base', local_files_only=True):
        self.model = WhisperModel(
            model_size_or_path=model_size_or_path,
            local_files_only=local_files_only,
        )

    def audio_to_srt(self, audio, filename='subtitle.srt', **transcribe_kwargs):
        segments = self.transcribe(audio, **transcribe_kwargs)
        self.to_srt(segments, filename)

    def transcribe(
            self,
            audio,
            language: Optional[str] = 'zh',
            task: str = "transcribe",
            **kwargs  # todo: 常用参数
    ):
        segments, self.info = self.model.transcribe(str(audio), language=language, task=task, )

        bn = BasicTextNormalizer()

        bar = tqdm(segments)
        for segment in bar:
            d = segment._asdict()
            d['text'] = bn(chinese_convert(segment.text))
            segment = Segment(**d)

            bar.set_description(segment.text)
            yield segment

    def to_srt(self, segments: Generator[Segment, Any, None], filename='subtitle.srt'):
        subs = pysrt.SubRipFile()

        for index, segment in enumerate(segments):  # todo: index 从0开始？
            subs.append(
                pysrt.SubRipItem(
                    index=index,
                    start=self.seconds_to_srt_time(segment.start),
                    end=self.seconds_to_srt_time(segment.end),
                    text=segment.text
                )
            )

            # if index > 2: break

        subs.save(filename, encoding='utf-8')

    @staticmethod
    @lru_cache
    def seconds_to_srt_time(seconds):
        """
            # 示例用法
            seconds = 350.00
            formatted_srt_time = format_seconds_to_srt_time(seconds)
            print(formatted_srt_time)
        :param seconds:
        :return:
        """
        # 使用 timedelta 类型创建时间间隔对象
        time_interval = timedelta(seconds=seconds)

        # 创建一个 SubRipTime 对象
        srt_time = pysrt.SubRipTime(
            hours=time_interval.seconds // 3600,
            minutes=(time_interval.seconds // 60) % 60,
            seconds=time_interval.seconds % 60,
            milliseconds=int(time_interval.microseconds / 1000)
        )

        return srt_time


if __name__ == '__main__':
    model_size = "small"
    # model_size = "large-v3"
    # model_size = "Systran/faster-distil-whisper-large-v3"
    asr = ASR(model_size_or_path=model_size, local_files_only=True)
    # audio_path = '../../../../zh_.wav'
    # audio_path = "/Users/betterme/Downloads/videoplayback.m4a"
    # audio_path = "20190101-section_2.mp3"
    # asr.audio_to_srt(audio_path)

    for p in Path('./').glob('*.mp3'):
        asr.audio_to_srt(
            p, f"./{p.parent.name}/{p.name.strip('.mp3')}.text",
        )
        break



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : conf
# @Time         : 2021/1/31 10:20 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *


# 定义参数
class TrainConf(BaseConfig):
    epoch = 10
    batch_size = 128


def train(**kwargs):
    logger.info("开始训练")
    time.sleep(3)


# 使用参数
def run(**kwargs):
    logger.info(f"输入参数: {kwargs}")
    c = TrainConf.parse_obj(kwargs)
    logger.info(f"使用参数: {c.dict()}")
    train(**c.dict())


# 传入参数
conf_cli = lambda: fire.Fire(run)  # <conf_cli> --epoch 11 --batch_size 111
# fire.Fire()需要指定命令对象



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : text_to_image
# @Time         : 2024/7/8 12:19
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://docs.siliconflow.cn/reference/stabilityaistable-diffusion-3-medium_text-to-image

from meutils.pipe import *
from meutils.pipe import storage_to_cookie
from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.apis.translator import deeplx
from meutils.schemas.translator_types import DeeplxRequest
from meutils.decorators.retry import retrying
from meutils.schemas.image_types import ASPECT_RATIOS

BASE_URL = "https://api.siliconflow.cn/v1"
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=InxiCF"
FEISHU_URL_TOKEN = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=xlvlrH"


@retrying(max_retries=5, title=__name__)
async def create(request: ImageRequest, api_key: Optional[str] = None):  # SD3
    api_key = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    url = f"{request.model}/text-to-image"
    if request.model.startswith(('black-forest-labs',)):  # 'black-forest-labs/FLUX.1-dev'
        if 'dev' in request.model:
            url = "/image/generations"
    else:
        request.prompt = (await deeplx.translate(DeeplxRequest(text=request.prompt, target_lang="EN"))).get("data")

    # if not request.prompt_enhancement:
    #     request.prompt = (await deeplx.translate(DeeplxRequest(text=request.prompt, target_lang="EN"))).get("data")
    payload = {
        "model": request.model,
        "prompt": request.prompt,
        "image_size": ASPECT_RATIOS.get(request.size, request.size),
        "batch_size": request.n,
        "num_inference_steps": request.num_inference_steps,
        "guidance_scale": request.guidance_scale,
        "prompt_enhancement": True
    }

    headers = {
        "Authorization": f"Bearer {api_key}",
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:

        response = await client.post(url, json=payload)

        logger.debug(response.json())

        if response.is_success:
            data = response.json().get('images', [])
            return ImagesResponse(data=data)
        raise response.raise_for_status()  # 451
        # from fastapi import HTTPException, status

        # raise HTTPException(status_code=response.status_code, detail=response.text) from response.raise_for_status()


if __name__ == '__main__':
    # cookie = await get_next_token_for_polling(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=xlvlrH")
    # api_key = storage_to_cookie(cookie)

    # storage_state = get_spreadsheet_values(feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=xlvlrH",
    #                            to_dataframe=True)[0]

    # cookie = storage_to_cookie(storage_state[0])

    # request = ImageRequest(
    #     # model="stabilityai/stable-diffusion-3-medium",
    #     # model="black-forest-labs/FLUX.1-schnell",
    #     model="black-forest-labs/FLUX.1-dev",
    #
    #     prompt="一条可爱的狗"
    # )
    # print(arun(create_image(request)))

    # arun(get_next_token_for_polling(feishu_url=FEISHU_URL_TOKEN))
    # request = ImageRequest(
    #     prompt="画条狗",
    #     model="stabilityai/stable-diffusion-3-medium"
    # )
    #
    # with timer():
    #     arun(
    #         api_create_image(
    #             request
    #         )
    #     )
    #
    # 内容审核测试
    prompt = """
    一个中国美女，在河边
    """
    request = ImageRequest(
        prompt=prompt,
        model="black-forest-labs/FLUX.1-dev",
        # model="black-forest-labs/FLUX.1-schnell",
        # model="ByteDance/SDXL-Lightning",

        # size='1366x1366',
        n=1
    )

    with timer():

        try:
            arun(
                create(
                    request,
                )
            )
        except Exception as e:
            print(e)
    # request = ImageRequest(
    #     prompt=prompt,
    #     model="black-forest-labs/FLUX.1-dev"
    # )
    #
    # with timer():
    #     arun(
    #         api_create_image(
    #             request
    #         )
    #     )



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

def print_labels():

    print(
        "LABELS INFO:\n\n",
        "prob_min         : Minimum probability in a particular decile\n",
        "prob_max         : Minimum probability in a particular decile\n",
        "prob_avg         : Average probability in a particular decile\n",
        "cnt_events       : Count of events in a particular decile\n",
        "cnt_resp         : Count of responders in a particular decile\n",
        "cnt_non_resp     : Count of non-responders in a particular decile\n",
        "cnt_resp_rndm    : Count of responders if events assigned randomly in a particular decile\n",
        "cnt_resp_wiz     : Count of best possible responders in a particular decile\n",
        "resp_rate        : Response Rate in a particular decile [(cnt_resp/cnt_cust)*100]\n",
        "cum_events       : Cumulative sum of events decile-wise \n",
        "cum_resp         : Cumulative sum of responders decile-wise \n",
        "cum_resp_wiz     : Cumulative sum of best possible responders decile-wise \n",
        "cum_non_resp     : Cumulative sum of non-responders decile-wise \n",
        "cum_events_pct   : Cumulative sum of percentages of events decile-wise \n",
        "cum_resp_pct     : Cumulative sum of percentages of responders decile-wise \n",
        "cum_resp_pct_wiz : Cumulative sum of percentages of best possible responders decile-wise \n",
        "cum_non_resp_pct : Cumulative sum of percentages of non-responders decile-wise \n",
        "KS               : KS Statistic decile-wise \n",
        "lift             : Cumuative Lift Value decile-wise",
         )



def decile_table(y_true, y_prob, change_deciles=10, labels=True, round_decimal=3):
    """Generates the Decile Table from labels and probabilities
    
    The Decile Table is creared by first sorting the customers by their predicted 
    probabilities, in decreasing order from highest (closest to one) to 
    lowest (closest to zero). Splitting the customers into equally sized segments, 
    we create groups containing the same numbers of customers, for example, 10 decile 
    groups each containing 10% of the customer base.
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct/actual) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier/algorithm.

        change_deciles (int, optional): The number of partitions for creating the table
            can be changed. Defaults to '10' for deciles.

        labels (bool, optional): If True, prints a legend for the abbreviations of
            decile table column names. Defaults to True.

        round_decimal (int, optional): The decimal precision till which the result is 
            needed. Defaults to '3'.

    Returns:
        dt: The dataframe dt (decile-table) with the deciles and related information.

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.decile_table(y_test, y_prob[:,1])
    """
    y_true = np.array(y_true)
    y_prob = np.array(y_prob)

    df = pd.DataFrame()
    df['y_true'] = y_true
    df['y_prob'] = y_prob
    # df['decile']=pd.qcut(df['y_prob'], 10, labels=list(np.arange(10,0,-1))) 
    # ValueError: Bin edges must be unique

    df.sort_values('y_prob', ascending=False, inplace=True)
    df['decile'] = np.linspace(1, change_deciles+1, len(df), False, dtype=int)

    # dt abbreviation for decile_table
    dt = df.groupby('decile').apply(lambda x: pd.Series([
        np.min(x['y_prob']),
        np.max(x['y_prob']),
        np.mean(x['y_prob']),
        np.size(x['y_prob']),
        np.sum(x['y_true']),
        np.size(x['y_true'][x['y_true'] == 0]),
    ],
        index=(["prob_min", "prob_max", "prob_avg",
                "cnt_cust", "cnt_resp", "cnt_non_resp"])
    )).reset_index()

    dt['prob_min']=dt['prob_min'].round(round_decimal)
    dt['prob_max']=dt['prob_max'].round(round_decimal)
    dt['prob_avg']=round(dt['prob_avg'],round_decimal)
    # dt=dt.sort_values(by='decile',ascending=False).reset_index(drop=True)

    tmp = df[['y_true']].sort_values('y_true', ascending=False)
    tmp['decile'] = np.linspace(1, change_deciles+1, len(tmp), False, dtype=int)

    dt['cnt_resp_rndm'] = np.sum(df['y_true']) / change_deciles
    dt['cnt_resp_wiz'] = tmp.groupby('decile', as_index=False)['y_true'].sum()['y_true']

    dt['resp_rate'] = round(dt['cnt_resp'] * 100 / dt['cnt_cust'], round_decimal)
    dt['cum_cust'] = np.cumsum(dt['cnt_cust'])
    dt['cum_resp'] = np.cumsum(dt['cnt_resp'])
    dt['cum_resp_wiz'] = np.cumsum(dt['cnt_resp_wiz'])
    dt['cum_non_resp'] = np.cumsum(dt['cnt_non_resp'])
    dt['cum_cust_pct'] = round(dt['cum_cust'] * 100 / np.sum(dt['cnt_cust']), round_decimal)
    dt['cum_resp_pct'] = round(dt['cum_resp'] * 100 / np.sum(dt['cnt_resp']), round_decimal)
    dt['cum_resp_pct_wiz'] = round(dt['cum_resp_wiz'] * 100 / np.sum(dt['cnt_resp_wiz']), round_decimal)
    dt['cum_non_resp_pct'] = round(
        dt['cum_non_resp'] * 100 / np.sum(dt['cnt_non_resp']), round_decimal)
    dt['KS'] = round(dt['cum_resp_pct'] - dt['cum_non_resp_pct'], round_decimal)
    dt['lift'] = round(dt['cum_resp_pct'] / dt['cum_cust_pct'], round_decimal)

    if labels is True:
        print_labels()

    return dt



def plot_lift(y_true, y_prob, title='Lift Plot', title_fontsize=14,
              text_fontsize=10, figsize=None):
    """Generates the Decile based cumulative Lift Plot from labels and probabilities

    The lift curve is used to determine the effectiveness of a
    binary classifier. A detailed explanation can be found at
    http://www2.cs.uregina.ca/~dbd/cs831/notes/lift_chart/lift_chart.html
    The implementation here works only for binary classification.
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier.

        title (string, optional): Title of the generated plot. Defaults to
            "Lift Plot".

        title_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 14.

        text_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 10.        

        figsize (2-tuple, optional): Tuple denoting figure size of the plot
            e.g. (6, 6). Defaults to ``None``.

    Returns:
        None

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.plot_lift(y_test, y_prob[:,1])
    """

    # Cumulative Lift Plot
    # plt.subplot(2, 2, 1)

    pl = decile_table(y_true,y_prob,labels=False)
    plt.plot(pl.decile.values, pl.lift.values, marker='o', label='Model')
    # plt.plot(list(np.arange(1,11)), np.ones(10), 'k--',marker='o')
    plt.plot([1, 10], [1, 1], 'k--', marker='o', label='Random')
    plt.title(title, fontsize=title_fontsize)
    plt.xlabel('Deciles', fontsize=text_fontsize)
    plt.ylabel('Lift', fontsize=text_fontsize)
    plt.legend()
    plt.grid(True)
    # plt.show()



def plot_lift_decile_wise(y_true, y_prob, title='Decile-wise Lift Plot',
                          title_fontsize=14, text_fontsize=10, figsize=None):
    """Generates the Decile-wise Lift Plot from labels and probabilities

    The lift curve is used to determine the effectiveness of a
    binary classifier. A detailed explanation can be found at
    http://www2.cs.uregina.ca/~dbd/cs831/notes/lift_chart/lift_chart.html
    The implementation here works only for binary classification.
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier.

        title (string, optional): Title of the generated plot. Defaults to
            "Decile-wise Lift Plot".

        title_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 14.

        text_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 10.

        figsize (2-tuple, optional): Tuple denoting figure size of the plot
            e.g. (6, 6). Defaults to ``None``.

    Returns:
        None

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.plot_lift_decile_wise(y_test, y_prob[:,1])
    """
    # Decile-wise Lift Plot
    # plt.subplot(2, 2, 2)
    pldw = decile_table(y_true,y_prob,labels=False)
    plt.plot(pldw.decile.values, pldw.cnt_resp.values / pldw.cnt_resp_rndm.values, marker='o', label='Model')
    # plt.plot(list(np.arange(1,11)), np.ones(10), 'k--',marker='o')
    plt.plot([1, 10], [1, 1], 'k--', marker='o', label='Random')
    plt.title(title, fontsize=title_fontsize)
    plt.xlabel('Deciles', fontsize=text_fontsize)
    plt.ylabel('Lift @ Decile', fontsize=text_fontsize)
    plt.legend()
    plt.grid(True)
    # plt.show()



def plot_cumulative_gain(y_true, y_prob, title='Cumulative Gain Plot',
                         title_fontsize=14, text_fontsize=10, figsize=None):
    """Generates the cumulative Gain Plot from labels and probabilities

    The cumulative gains chart is used to determine the effectiveness of a
    binary classifier. A detailed explanation can be found at
    http://www2.cs.uregina.ca/~dbd/cs831/notes/lift_chart/lift_chart.html 
    The implementation here works only for binary classification.
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier.

        title (string, optional): Title of the generated plot. Defaults to
            "Decile-wise Lift Plot".

        title_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 14.

        text_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 10.

        figsize (2-tuple, optional): Tuple denoting figure size of the plot
            e.g. (6, 6). Defaults to ``None``.

    Returns:
        None

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.plot_cumulative_gain(y_test, y_prob[:,1])
    """

    # Cumulative Gains Plot
    # plt.subplot(2, 2, 3)
    pcg = decile_table(y_true,y_prob,labels=False)
    plt.plot(np.append(0, pcg.decile.values), np.append(0, pcg.cum_resp_pct.values), marker='o', label='Model')
    plt.plot(np.append(0, pcg.decile.values), np.append(0, pcg.cum_resp_pct_wiz.values), 'c--', label='Wizard')
    # plt.plot(list(np.arange(1,11)), np.ones(10), 'k--',marker='o')
    plt.plot([0, 10], [0, 100], 'k--', marker='o', label='Random')
    plt.title(title, fontsize=title_fontsize)
    plt.xlabel('Deciles', fontsize=text_fontsize)
    plt.ylabel('% Resonders', fontsize=text_fontsize)
    plt.legend()
    plt.grid(True)
    # plt.show()



def plot_ks_statistic(y_true, y_prob, title='KS Statistic Plot',
                      title_fontsize=14, text_fontsize=10, figsize=None):
    """Generates the KS Statistic Plot from labels and probabilities

    Kolmogorov-Smirnov (KS) statistic is used to measure how well the 
    binary classifier model separates the Responder class (Yes) from 
    Non-Responder class (No). The range of K-S statistic is between 0 and 1. 
    Higher the KS statistic value better the model in separating the 
    Responder class from Non-Responder class.
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier.

        title (string, optional): Title of the generated plot. Defaults to
            "KS Statistic Plot".

        title_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 14.

        text_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 10.

        figsize (2-tuple, optional): Tuple denoting figure size of the plot
            e.g. (6, 6). Defaults to ``None``.

    Returns:
        None

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.plot_ks_statistic(y_test, y_prob[:,1])
    """
    # KS Statistic Plot
    # plt.subplot(2, 2, 4)
    pks = decile_table(y_true, y_prob, labels=False)

    plt.plot(np.append(0, pks.decile.values), np.append(0, pks.cum_resp_pct.values),
             marker='o', label='Responders')
    plt.plot(np.append(0, pks.decile.values), np.append(0, pks.cum_non_resp_pct.values),
             marker='o', label='Non-Responders')
    # plt.plot(list(np.arange(1,11)), np.ones(10), 'k--',marker='o')
    ksmx = pks.KS.max()
    ksdcl = pks[pks.KS == ksmx].decile.values
    plt.plot([ksdcl, ksdcl],
             [pks[pks.KS == ksmx].cum_resp_pct.values,
              pks[pks.KS == ksmx].cum_non_resp_pct.values],
             'g--', marker='o', label='KS Statisic: ' + str(ksmx) + ' at decile ' + str(list(ksdcl)[0]))
    plt.title(title, fontsize=title_fontsize)
    plt.xlabel('Deciles', fontsize=text_fontsize)
    plt.ylabel('% Resonders', fontsize=text_fontsize)
    plt.legend()
    plt.grid(True)
    # plt.show()



def report(y_true, y_prob, labels=True, plot_style = 'seaborn', round_decimal=3,
           title_fontsize=14, text_fontsize=10, figsize=(16, 10)):
    """Generates decile table and 4 plots (Lift, Lift@Decile, Gain and KS) 
    from labels and probabilities
    
    Args:
        y_true (array-like, shape (n_samples)):
            Ground truth (correct) target values.

        y_prob (array-like, shape (n_samples, n_classes)):
            Prediction probabilities for each class returned by a classifier.

        labels (bool, optional): If True, prints a legend for the abbreviations of
            decile table column names. Defaults to True.

        plot_style(string, optional): Check available styles "plt.style.available".
            few examples: ['ggplot', 'seaborn', 'bmh', 'classic', 'dark_background', 
            'fivethirtyeight', 'grayscale', 'seaborn-bright', 'seaborn-colorblind', 
            'seaborn-dark', 'seaborn-dark-palette', 'tableau-colorblind10','fast'] 
            Defaults to ``None``.

        round_decimal (int, optional): The decimal precision till which the result is 
            needed. Defaults to '3'.

        title_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 14.

        text_fontsize (string or int, optional): Matplotlib-style fontsizes.
            Use e.g. "small", "medium", "large" or integer-values (8, 10, 12, etc.)
            Defaults to 10.

        figsize (2-tuple, optional): Tuple denoting figure size of the plot
            e.g. (6, 6). Defaults to ``None``.

    Returns:
        dc: The dataframe dc (decile-table) with the deciles and related information.

    Example:
        >>> import kds
        >>> from sklearn.datasets import load_iris
        >>> from sklearn.model_selection import train_test_split
        >>> from sklearn import tree
        >>> X, y = load_iris(return_X_y=True)
        >>> X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.33,random_state=3)
        >>> clf = tree.DecisionTreeClassifier(max_depth=1,random_state=3)
        >>> clf = clf.fit(X_train, y_train)
        >>> y_prob = clf.predict_proba(X_test)
        >>> kds.metrics.report(y_test, y_prob[:,1])
    """

    dc = decile_table(y_true,y_prob,labels=labels,round_decimal=round_decimal)

    if plot_style is None:
        None
    else:
        plt.style.use(plot_style)

    fig = plt.figure(figsize=figsize)

    # Cumulative Lift Plot
    plt.subplot(2, 2, 1)
    plot_lift(y_true,y_prob)

    #  Decile-wise Lift Plot
    plt.subplot(2, 2, 2)
    plot_lift_decile_wise(y_true,y_prob)

    # Cumulative Gains Plot
    plt.subplot(2, 2, 3)
    plot_cumulative_gain(y_true,y_prob)

    # KS Statistic Plot
    plt.subplot(2, 2, 4)
    plot_ks_statistic(y_true,y_prob)

    return (dc)


import requests
import time
import os

# 上传图片
def upload_image(token, image_path, purpose="vidu", vip=False):
    url = "https://api.chatfire.cn/v1/files"
    headers = {
        'Authorization': f'Bearer {token}',
    }
    files = {
        'file': open(image_path, 'rb'),
        'purpose': (None, purpose)
    }
    params = {
        'vip': str(vip).lower()
    }
    response = requests.post(url, headers=headers, files=files, params=params)
    
    if response.status_code == 200:
        print("图片上传成功")
        return response.json()  # 返回响应内容，包括图片的 URL
    else:
        print(f"图片上传失败，状态码: {response.status_code}, 错误信息: {response.text}")
        return None

# 创建图生视频任务
def create_img_to_video_task(token, prompt, style, aspect_ratio, duration, image_url):
    url = "https://api.chatfire.cn/tasks/vidu?vip=true"
    headers = {
        'Authorization': f'Bearer {token}',
        'Content-Type': 'application/json'
    }
    payload = {
        "prompt": prompt,
        "style": style,
        "aspect_ratio": aspect_ratio,
        "duration": duration,
        "url": image_url
    }
    response = requests.post(url, json=payload, headers=headers)
    
    if response.status_code == 200:
        print("图生视频任务创建成功")
        return response.json()  # 返回任务ID
    else:
        print(f"图生视频任务创建失败，状态码: {response.status_code}, 错误信息: {response.text}")
        return None

# 获取任务状态
def get_task_status(token, task_id):
    url = f"https://api.chatfire.cn/tasks/{task_id}"
    headers = {
        'Authorization': f'Bearer {token}',
    }
    response = requests.get(url, headers=headers)
    
    if response.status_code == 200:
        return response.json()
    else:
        print(f"获取任务状态失败，状态码: {response.status_code}, 错误信息: {response.text}")
        return None

# 下载视频
def download_video(video_url, save_path):
    response = requests.get(video_url)
    if response.status_code == 200:
        with open(save_path, 'wb') as file:
            file.write(response.content)
        print(f"视频下载成功: {save_path}")
    else:
        print(f"视频下载失败，状态码: {response.status_code}, 错误信息: {response.text}")

# 主流程：图片上传 -> 图生视频 -> 获取任务状态并下载视频
def process_image_to_video(token, image_path, prompts, style, aspect_ratio, duration, save_video_path):
    # Step 1: 上传图片
    upload_result = upload_image(token, image_path)
    if not upload_result:
        return

    image_url = upload_result.get('url')
    if not image_url:
        print("未能获取到图片URL")
        return

    # Step 2: 遍历所有 prompt 并为每个 prompt 创建图生视频任务
    for i, prompt in enumerate(prompts):
        print(f"\n开始处理第 {i+1} 个任务: {prompt}\n")
        
        video_task = create_img_to_video_task(token, prompt, style, aspect_ratio, duration, image_url)
        if not video_task:
            continue

        task_id = video_task.get('id')
        if not task_id:
            print("未能获取到任务ID")
            continue

        # Step 3: 轮询获取任务状态
        print(f"任务 {task_id} 创建成功，正在等待视频生成...")
    
        while True:
            try:
                task_status = get_task_status(token, task_id)
                if not task_status:
                    break
                
                state = task_status.get('state')
                if state == 'success':
                    print("视频生成成功")
                    video_url = task_status['creations'][0]['uri']  # 获取生成的视频URL
                    # 下载视频
                    download_video(video_url, f"{save_video_path}_{i+1}.mp4")
                    break
                elif state == 'failed':
                    print("视频生成失败")
                    break
                else:
                    print(f"当前任务状态: {state}，继续等待...")
                
                time.sleep(5)  # 每5秒检查一次状态
            
            except requests.exceptions.ConnectionError:
                print("远程主机强行关闭连接，重新尝试...")
                time.sleep(5)  # 等待5秒后重新尝试
            except requests.exceptions.RequestException as e:
                print(f"发生错误: {e}")
                break

# 示例调用
API_key = "sk-oEDQTphWyIXesuau947f40Cd7dE34692B685C92a1056651c"
image_path = "E:\\Pet_AI_Empower\\voive\\dog_dataset\\bg.jpg"
style = "general"
aspect_ratio = "16:9"
duration = 4
save_video_path = "bg_prompt_copy"  # 不带扩展名，以便区分不同视频

# 描述词
prompts = [
    #"乖巧地坐在地上",11111111111111
    #"静静地将后腿折叠在身体下",
    #"优雅地坐在主人的面前",
    #"低头，后腿自然弯曲",
    #"稳定地坐在草地上",
    
    #"将身体舒展在地面上",
    #"前腿伸直，后腿弯曲",
    #"头轻轻地靠在爪子上",
    #"全身紧贴地面，尾巴微微摆动",
    #"放松地趴在阳光下",11111111111
    
    #"高高跃起，四肢悬空",2
    #"敏捷地从地面弹起",1
    #"空中优雅地弯曲身体",
    #"腾空跳跃，耳朵随着动作摆动",3
    #"轻盈地跳向前方",(有点像跑的)
    
    #"前爪伸出，轻触主人的手",
    #"举起前腿，稳稳地与主人握手",1
    #"用前爪轻轻地按住主人的手掌",
    #"抬起一只前爪，期待主人的回应",
    #"友好地将爪子放在主人的手中",
    
    #"狗狗完全趴在地上，前爪伸展，头部贴近地面。它先抬起头，前爪用力，将上半身慢慢撑离地面，后腿逐渐发力抬起身体，最后狗狗完全站立起来，姿势稳固。",
    #"狗狗从完全趴下的状态开始，先是抬起头部，前爪用力将上半身撑起，随后后腿发力，缓慢站起来，最后身体直立，站稳在原地。",#坐下的状态
    #"狗狗趴在地上，前爪向前撑，身体前半部分抬离地面，后腿紧跟着抬起，狗狗的身体逐渐离开地面，慢慢站直，最后完全站立起来。",
    #"狗狗趴在地面上，头部靠近地面。它先用前爪撑地，抬起上半身，接着后腿用力将身体从地面抬起，狗狗慢慢从趴下状态站起来，最终直立。",
    #"狗狗趴在地上，前爪和后腿伸展。它先抬起头，前爪用力将上身推起，然后后腿逐步用力，整个身体从地面站起，最终站得笔直，姿势稳定。",    #有趴下状态，但是站立动作不自然
    #"狗狗安静地趴在地上，先抬头并用前爪撑起前身，后腿慢慢从地面上抬起，身体逐渐直立，最后完全站立，动作流畅自然。",
    #"狗狗趴在地上，前爪开始用力撑起上身，后腿随后发力，整个身体从地面抬起，最后站直在原地，狗狗姿态平稳。",
    
    
    #"期待地举起前爪，进行握手动作",  #32
    #"稳定地将爪子放在主人的手上，表现出亲密",
    #"轻轻用爪子拍打主人的手，显示友善",    
    #"从坐着的姿势中抬起前腿，慢慢站起" ,
    #"逐渐从坐下的位置转变为站立，展现活力" ,
    #"从趴下的姿势中抬起前爪，身体向上站起" ,
    #"缓缓起身，从舒展的趴姿转为稳健的站立" ,#38
    
    #"友好地抬起前爪，轻轻搭在主人的手上，展现亲昵",#39
    #"将前腿伸出，主动与主人握手，表达亲密互动",
    #"稳定地用爪子轻触主人的手，显示出信任和依赖",
    #"期待地抬起前爪，与主人手心相贴，完成握手",
    #"俏皮地用前爪拍打主人的手，展示出活泼和友好",
    #"从坐着的姿势中，前爪先抬起，优雅地转为站立",
    #"从趴下的姿势中，先抬起头，然后用前腿支撑身体起立",
    #"缓缓起身，前腿稳稳地撑起，尾巴高高翘起，显示自信",#失败了    #下面这个好
    #"狗坐在主人面前，耳朵竖起，目光专注。它抬起一只前爪，轻轻搭在主人的手掌上，仿佛在说“你好！”狗狗的表情愉悦，尾巴微微摇摆，展示出亲密和友好。主人微笑着回应，轻轻握住狗狗的爪子，彼此的互动让这一刻充满温馨。",
    #下面这条趴下
    #"狗狗舒适地趴在地上，前爪自然地伸展，耳朵柔软地垂下，显得十分放松。随着主人温柔的召唤声，狗狗的耳朵迅速竖起，眼睛睁大，流露出兴奋与期待。它开始先抬起一只前爪，慢慢地将爪子推向地面，准备支撑起身体。接着，狗狗微微弓起背部，后腿用力，前腿也随之向下用力，身体向上倾斜。尾巴高高翘起，像个小旗子一样摇摆，展现出它的活力与兴奋。最后，狗狗的前腿稳稳落在地面上，后腿支撑起整个身体，缓缓站直，目光坚定地注视着主人，似乎在说：“我准备好迎接新的一天了！"
    ####"狗狗懒洋洋地趴在地上，前爪轻轻伸展，耳朵垂下，显得十分放松。听到主人的召唤声，狗狗的耳朵瞬间竖起，眼中闪烁着兴奋的光芒。它先将一只前爪缓缓抬起，轻触地面，随后用力撑起前身，背部微微弓起。狗狗的后腿开始紧绷，随着身体的倾斜，它开始努力地将后腿推开，慢慢向上站起。尾巴兴奋地摇摆，仿佛在为这个动作加油。最后，狗狗稳稳地站起来，目光直视前方，身体挺直，充满活力，像是在迎接新的冒险。",
    #"狗狗懒洋洋地趴在地上，前爪自然放松，耳朵垂下，显得十分惬意。随着主人柔和的召唤声，狗狗的耳朵立刻竖起，眼中闪现出兴奋的光芒。它缓缓抬起一只前爪，轻轻按在地面上，随后用力撑起身体，背部微微弓起，显示出努力的样子。后腿开始用力，稳稳地推开地面，逐渐站起。狗狗的尾巴高高翘起，兴奋地摇摆着，传达出它的快乐与期待。当它完全站起来时，四肢稳稳地落在地面上，身体挺直，目光坚定地注视着主人，似乎在说：“我准备好了，随时出发！”这一刻，狗狗散发出满满的自信与活力，整个过程流畅自然，没有任何浮动的感觉。"
    #"狗狗舒适地趴在地上，前爪自然放松，耳朵轻轻垂下，显得惬意而放松。听到主人的召唤，狗狗的耳朵瞬间竖起，眼中闪烁着兴奋的光芒。它首先微微抬起头，接着一只前爪慢慢按在地面上，准备支撑身体。然后，狗狗用力推开地面，缓缓弓起背部，开始站立。后腿紧绷，逐渐将身体完全支撑起来，尾巴兴奋地摇摆，仿佛在为这一动作欢呼。站立时，狗狗目光坚定，向前注视，散发出自信的气息。此时，它的前爪轻轻搭在主人伸出的手上，展现出亲密的互动，似乎在请求主人给予鼓励和赞赏。",
    #"趴下后坐起来"

    #"A standing dog gradually sits down, moving naturally with a smooth transition.",
    "一只站着的狗逐渐坐下，动作自然，过渡流畅",
    
    #"The dog sits quietly in front of its owner, with a gentle look in its eyes.",
    "狗狗安静地坐在主人面前，眼神温柔。",
    
    #"Sitting obediently on the ground.",
    "乖巧地坐在地上",
    
    #"A dog slowly lies down on the ground, first bending its front legs as its body lowers closer to the ground, front paws gently stretched out, then its back legs follow, lying flat. Finally, it lowers its head and fully relaxes, lying peacefully.",
    "一只狗在地上慢慢趴下，先是弯曲前腿，身体逐渐贴近地面，前爪轻轻伸展，然后后腿也跟着平铺在地上，头部放低，最后完全放松地趴在地上。",
    
    #"Starting from a lying position, the dog pushes itself up with its front paws, slowly lifting its back legs, gracefully rising to stand with steady posture.c",
    "狗狗从趴着姿势开始，前爪用力撑起身体，后腿慢慢抬起，狗狗缓缓站立起来，姿势优雅且稳固。",
    
    #"Relaxing in the sunlight, lying peacefully",
    "放松地趴在阳光下",
    
    #"The dog springs up from the ground with agility, all four legs in the air.",
    "狗狗敏捷地从地面弹起，四肢悬空。",
    
    #"Quickly springing up from the ground.",
    "敏捷地从地面弹起",
    
    #"Leaping high into the air, all four paws off the ground.",
    "高高跃起，四肢悬空",
    
    #"The dog places its paw gently in its owner's hand in a friendly gesture.",
    "友好地将爪子放在主人的手中",
    
    #"Raising its front leg, the dog firmly shakes hands with its owner.",
    "举起前腿，稳稳地与主人握手",
    
    #"The dog sits in front of its owner, ears perked up and eyes focused. It lifts a front paw and gently places it on the owner’s hand, as if saying ‘hello!’ The dog’s expression is happy, and its tail wags slightly, showing affection and friendliness. The owner smiles back and lightly grips the dog’s paw, making the moment warm and full of connection.",
    "狗坐在主人面前，耳朵竖起，目光专注。它抬起一只前爪，轻轻搭在主人的手掌上，仿佛在说“你好！”狗狗的表情愉悦，尾巴微微摇摆，展示出亲密和友好。主人微笑着回应，轻轻握住狗狗的爪子，彼此的互动让这一刻充满温馨。",
    
    #"The dog, lying down, pushes up with its front legs, then gradually uses its back legs to lift its whole body, slowly transitioning from lying to standing, ears perking up slightly.",#"狗狗趴着，先用前腿撑起身体，后腿逐步发力将整个身体抬起，狗狗慢慢从趴姿变为站立状态，耳朵微微竖起。",
    "狗躺下后，用前腿将身体撑起，然后逐渐用后腿将整个身体抬起，慢慢地从躺着过渡到站着，耳朵微微竖起。",        
    
    #"The dog lies lazily on the ground, front paws gently stretched out, ears drooping, looking completely relaxed. As soon as it hears its owner’s call, its ears perk up instantly, and excitement lights up its eyes. It slowly lifts one front paw, lightly touching the ground, then pushes up with effort, arching its back slightly. Its hind legs tense as its body leans forward, and it begins to push off with its back legs, slowly rising to a stand. Its tail wags excitedly, almost cheering itself on. Finally, the dog stands firmly, eyes focused ahead, body straight and full of energy, as if ready to embrace a new adventure.",
    "狗狗懒洋洋地趴在地上，前爪轻轻伸展，耳朵垂下，显得十分放松。听到主人的召唤声，狗狗的耳朵瞬间竖起，眼中闪烁着兴奋的光芒。它先将一只前爪缓缓抬起，轻触地面，随后用力撑起前身，背部微微弓起。狗狗的后腿开始紧绷，随着身体的倾斜，它开始努力地将后腿推开，慢慢向上站起。尾巴兴奋地摇摆，仿佛在为这个动作加油。最后，狗狗稳稳地站起来，目光直视前方，身体挺直，充满活力，像是在迎接新的冒险。",        
    
    #00000000000
    
    #"一只活力满满的狗狗轻快地跳起，四肢离地，耳朵随动作飘动，脸上带着兴奋的神情。",
    #"A lively dog jumps gracefully into the air, all four legs off the ground, ears flapping with the motion, and an excited expression on its face.",  
    #"A lively dog leaps into the air, all four paws off the ground, ears flapping with the movement, and an excited expression on its face.",            #111111111
    #"这只黑白边牧优雅地趴在地上，前肢伸直，头微微低垂，目光专注地注视着前方，等待主人指令。",
    #"The dog elegantly lies down on the ground, front legs stretched out, head slightly lowered, and its eyes focused intently ahead, waiting for a command.",
    #"The dog gracefully lies down, front legs stretched out, head slightly lowered, and eyes focused intently ahead, waiting for its owner’s command.",   #00000000000
    #"狗狗用后腿站立，前爪轻轻抬起，身体平衡，眼睛闪烁着对周围环境的好奇。",
    #"The dog up on its hind legs, front paws raised slightly, maintaining balance, with eyes gleaming in curiosity as it surveys its surroundings.",
    #"狗狗坐着，伸出一只前爪与主人握手，尾巴轻轻摇摆，表现出友好与信任。",
    #"The dog sits and extends one of its front paws to shake hands with its owner, tail wagging gently, showing friendliness and trust.",
    #"The dog sits and extends a paw to shake hands with its owner, tail gently wagging, showing friendliness and trust.",                     #00000000000
    #"狗狗端坐在草地上，背部挺直，目光温和，神态安详，偶尔抬头看向远处。",
    #"The dog sits upright on the grass, back straight, eyes calm and gentle, occasionally glancing into the distance with a peaceful demeanor."
    #"The dog sits upright on the grass, back straight, with a calm and gentle gaze. It occasionally looks up into the distance, appearing peaceful and serene."   #
    
]

# 执行主流程
process_image_to_video(API_key, image_path, prompts, style, aspect_ratio, duration, save_video_path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : crawler
# @Time         : 2021/9/2 下午2:47
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : httpx异步优化
# @Description  : https://blog.csdn.net/u013332124/article/details/80621638
# pd: https://blog.csdn.net/zhang862520682/article/details/86701078

from lxml.etree import HTML
from meutils.request_utils import request, request4retry


class Crawler(object):

    def __init__(self, url, encoding=None, *args, **kwargs):
        self.url = url
        self.html = self.get_html(url, encoding)

    def xpath(self, _path="//text()", **_variables):
        return self.html.xpath(_path, **_variables)

    @staticmethod
    def get_html(url, encoding='utf-8'):
        r = request4retry(url, return_json=False, encoding=encoding)
        return HTML(r.text)


if __name__ == '__main__':
    from meutils.pipe import *
    url = "https://top.baidu.com/board?tab=realtime"

    #
    #
    # _ = Crawler(url).xpath('//*[@id="sanRoot"]/main/div[2]/div/div[2]/div[*]/div[2]/a/div[1]//text()')
    # print("\n".join(_))



    # from meutils.request_utils.crawler import Crawler
    #
    # url = 'http://10.2.41.63:8004/bert_models/bert-base-uncased/'
    # Crawler(url).xpath('//@href')

    # c = Crawler('https://www.maoyan.com/board?offset=0') # 猫眼top10
    # _ = c.xpath('//*[@id="app"]/div/div/div/dl/dd[*]/div/div/div[1]//text()') | xgroup(7)
    # top10 = pd.DataFrame(_)

    # from meutils.request_utils.crawler import Crawler
    # w = "宝能公馆1288"
    # url = f"https://m.fang.com/fangjia/sh_list_pinggu/?keyword={w}&city=sh&r=0.9648271001521231"
    #
    # print(Crawler(url).xpath('//*[@id="houselist"]/li/a//text()'))

    url  = "https://chat.tune.app/?id=7f268d94-d2d4-4bd4-a732-f196aa20dceb"
    url = "https://app.yinxiang.com/fx/8b8bba1e-b254-40ff-81e1-fa3427429efe"

    print(Crawler(url).xpath('//script//text()'))

    # html_content = httpx.get(url).text


    # # 正则表达式匹配以 "/_next/static/chunks/7116-" 开头的 JS 文件
    # pattern = r'(/_next/static/chunks/7116-[^"]+\.js)'
    #
    # # 使用 re.findall() 找到所有匹配项
    # matches = re.findall(pattern, html_content)
    #
    # "/_next/static/chunks/7116-aed224a0caaab94c.js"
    #
    # # 打印结果
    # for match in matches:
    #     print(match)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : replicateai
# @Time         : 2024/11/29 19:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.async_utils import async_to_sync_pro
from meutils.decorators.retry import retrying
from meutils.async_task import worker, shared_task

from meutils.apis.replicateai import raw as replicate


@shared_task(pydantic=True, retry_kwargs={'max_retries': 5, 'countdown': 10})
@async_to_sync_pro
async def create_task(request: replicate.ReplicateRequest, **kwargs):
    response = await replicate.create_task(request)
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/10/16 19:27
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://docs.together.ai/reference/post_images-generations


from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.llm.openai_utils import to_openai_images_params
from meutils.notice.feishu import IMAGES, send_message as _send_message
from meutils.apis.translator import deeplx

from meutils.schemas.translator_types import DeeplxRequest
from meutils.schemas.image_types import ImageRequest, TogetherImageRequest

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=tEsIyw"
FEISHU_URL_FREE = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=tEsIyw"

DEFAULT_MODEL = "black-forest-labs/FLUX.1-schnell-Free"
MODELS = {
    "flux-fast": DEFAULT_MODEL,
    "flux-turbo": DEFAULT_MODEL,
    "flux-schnell": DEFAULT_MODEL,

    "flux1.1-pro": "black-forest-labs/FLUX.1.1-pro",
}

send_message = partial(
    _send_message,
    title=__name__,
    url=IMAGES
)


async def generate(request: TogetherImageRequest, api_key: Optional[str] = None):
    request.model = MODELS.get(request.model, DEFAULT_MODEL)

    request.prompt = (
        await deeplx.translate(DeeplxRequest(text=request.prompt, target_lang="EN"))
    ).get("data", request.prompt)

    logger.debug(request)
    data = to_openai_images_params(request)

    for i in range(5):
        if any(i in request.model.lower() for i in {"pro"}):
            api_key = api_key or await get_next_token_for_polling(FEISHU_URL)
        else:
            api_key = api_key or await get_next_token_for_polling(FEISHU_URL_FREE, from_redis=True)

        try:
            client = AsyncOpenAI(base_url=os.getenv("TOGETHER_BASE_URL"), api_key=api_key)
            response = await client.images.generate(**data)
            response.model = ""
            return response
        except Exception as e:
            logger.error(e)
            if i > 2:
                send_message(f"生成失败: {e}\n\n{api_key}\n\n{request.model_dump_json(indent=4, exclude_none=True)}")


if __name__ == '__main__':
    from meutils.pipe import *

    api_keys = """
e3c18de174dc0cf7bdabc6f68432085a613fdabd2b3c2ffb4f3ed216f9aab44e
    """.split()

    for api_key in api_keys:
        request = TogetherImageRequest(model="pro", prompt="a dog", size="512x1024", n=4)
        print(request)
        arun(generate(request, api_key=api_key))
        break



import sys

import filetype


def guess(path):
    kind = filetype.guess(path)
    if kind is None:
        print('{}: File type determination failure.'.format(path))
    else:
        print('{}: {} ({})'.format(path, kind.extension, kind.mime))


def main():
    import argparse

    parser = argparse.ArgumentParser(
        prog='filetype', description='Determine type of FILEs.'
    )
    parser.add_argument('-f', '--file', nargs='+')
    parser.add_argument(
        '-v', '--version', action='version',
        version='%(prog)s ' + filetype.version,
        help='output version information and exit'
    )

    args = parser.parse_args()
    if len(sys.argv) < 2:
        parser.print_help()
        sys.exit(1)

    for i in args.file:
        guess(i)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : avmerge_
# @Time         : 2023/11/20 17:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

audio_path = "/Users/betterme/Downloads/videoplayback.m4a"
# audio_path = "audio.mp3"
video_path = "/Users/betterme/Downloads/videoplayback.mp4"


def merge_audio_video(audio_file, video_file, output_file):
    command = f'ffmpeg -i {audio_file} -i {video_file} -c:v copy -c:a aac -strict experimental {output_file}'
    subprocess.call(command, shell=True)


# 调用函数进行音频和视频合并
merge_audio_video(audio_path, video_path, "output.mp4")

#
# !ffmpeg -i /Users/betterme/Downloads/videoplayback.mp4 \
# -i audio_merged.wav \
# -vf "subtitles=new.srt" \
# -c:v libx264 -c:a aac \
# output.mp4



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : pay
# @Time         : 2024/4/29 16:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *



headers = {
    "Cookie": os.getenv("PAY_COOKIE"),
    "Content-Type": "application/x-www-form-urlencoded"
}

payload = {
    # "sitename": "",
    "trade_no": 2024042915220175845,
}

# print(httpx.post("https://zf.96ym.cn/User/Order/Callback", headers=headers, data=payload).text)


# df = pd.read_html("https://zf.96ym.cn/User/Order/Index", storage_options=headers)[0]
# df = df.query(f"`状态`!='已完成' and `商品名称`!='测试通道' and `创建时间 支付时间`>'{str(datetime.datetime.now())[:10]} 00:00:00'")
#
# print(df)

# """
# import httpx; httpx.post("https://zf.96ym.cn/User/Order/Callback", headers=headers, data=payload).text
#
# """




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/4/7 13:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : D3 生图、编辑图

from meutils.pipe import *

from meutils.apis.google.chat import Completions, CompletionRequest

from meutils.schemas.image_types import ImageRequest




async def generate(request: ImageRequest, api_key: Optional[str] = None):
    request = CompletionRequest(
        model="gemini-2.0-flash-exp-image-generation",
        messages=[

        ],
    )
    return Completions().create_for_images(request)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : seize
# @Time         : 2021/2/26 6:51 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : xgb/lgb 参数https://www.jianshu.com/p/1100e333fcab


# import wandb
#
# WANDB_API_KEY = '789ae399af943555652e476ff1d0c0452ee86564'
# wandb.login(key=WANDB_API_KEY)
#
# wandb.init()

from meutils.pipe import *
from xgboost import XGBClassifier
from sklearn.datasets import make_classification


def run(n_estimators=1000000):
    """

    :param n_estimators:
    :return:
    """

    X, y = make_classification(n_estimators)
    clf = XGBClassifier(learning_rate=1 / n_estimators, n_estimators=n_estimators, n_jobs=-1)

    try:
        clf.set_params(tree_method='gpu_hist', predictor='gpu_predictor')
        clf.fit(X, y, eval_set=[(X, y), (X, y)])

    except Exception as e:
        logger.warning(e)
        clf.fit(X, y, eval_set=[(X, y), (X, y)])


if __name__ == '__main__':
    with timer('seize'):
        run(100)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno
# @Time         : 2024/3/27 20:37
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


import jsonpath

from meutils.pipe import *
from meutils.schemas.task_types import Task
from meutils.schemas.suno_types import SunoAIRequest
from meutils.schemas.suno_types import MODELS, BASE_URL, CLIENT_BASE_URL, UPLOAD_BASE_UR, STUDIO_BASE_URL
from meutils.schemas.suno_types import API_SESSION, API_FEED, API_BILLING_INFO, API_GENERATE_LYRICS, API_GENERATE_V2

from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.io.files_utils import to_bytes

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=Jxlglo"
FEISHU_URL_STEM = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=rxldsA"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@alru_cache(ttl=3600)
# @retrying(max_retries=5, predicate=lambda r: not r)
async def get_refresh_token(token: str):  # 定时更新一次就行
    headers = {
        "Cookie": f"__client={token}"
    }
    async with httpx.AsyncClient(base_url=CLIENT_BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get('')

        # logger.debug(response.status_code)
        # logger.debug(response.text)

        if response.is_success:
            data = response.json()  # {"response":null,"client":null}
            if not data.get("response"):
                return token, None

            if ids := jsonpath.jsonpath(data, "$..last_active_session_id"):
                return token, ids[0]  # last_active_session_id

        send_message(f"未知错误：{response.status_code}\n\n{response.text}")


@alru_cache(ttl=30 - 3)
async def get_access_token(token: str):
    _ = await get_refresh_token(token)
    token, last_active_session_id = await get_refresh_token(token)  # last_active_token 没啥用

    headers = {
        "Cookie": f"__client={token}",
    }

    params = {
        "__clerk_api_version": "2021-02-05",
        "_clerk_js_version": "5.35.1"
    }

    async with httpx.AsyncClient(base_url=CLIENT_BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(f"/sessions/{last_active_session_id}/tokens", params=params)
        response.raise_for_status()
        logger.debug(bjson(response.json()))
        return response.json().get('jwt')

# /tokens?__clerk_api_version=2021-02-05&_clerk_js_version=5.35.1

@retrying(predicate=lambda r: not r)
async def create_task(request: SunoAIRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL, check_token=check_token, from_redis=True)

    access_token = await get_access_token(token)

    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    payload = request.model_dump(exclude_none=True)
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(API_GENERATE_V2, json=payload)
        if response.is_success:
            data = response.json()
            task_id, *clip_ids = jsonpath.jsonpath(data, "$..id")
            clip_ids = [i for i in clip_ids if not str(i).startswith("m_")]

            task_id = f"suno-{','.join(clip_ids)}"  # 需要返回的任务id
            return Task(id=task_id, data=data, system_fingerprint=token)

        response.raise_for_status()


@alru_cache(ttl=15)
@retrying(predicate=lambda r: not r)  # 触发重试
async def get_task(task_id, token: str):  # task_id 实际是 clip_ids， 必须指定token获取任务
    task_id = task_id.split("suno-")[-1]

    access_token = await get_access_token(token)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }

    params = {"ids": task_id}
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get(API_FEED, params=params)
        if response.is_success:
            return response.json()


@alru_cache(ttl=15)
@retrying(predicate=lambda r: not r)  # 触发重试
async def generate_lyrics(prompt: str = '', token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)

    access_token = await get_access_token(token)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }

    payload = {"prompt": prompt}

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(API_GENERATE_LYRICS, json=payload)
        if response.is_success:
            task_id = response.json().get("id")

            for i in range(100):
                response = await client.get(API_GENERATE_LYRICS + task_id)

                logger.debug(response.text)

                if response.is_success:
                    if response.json().get("status") == "complete":  # "status": "complete"
                        return response.json()  # todo: 大模型写歌词兜底
                    elif response.json().get("status") == "error":  # 退出
                        # send_message(f"生成歌词失败：{response.text}")
                        return "可能触发内容审核，请检查提示词"

                if i > 30:
                    response.raise_for_status()

                await asyncio.sleep(1 if i < 3 else 0.5)
        response.raise_for_status()


@alru_cache(ttl=3600)
async def upload(file: bytes, title: str = 'xx.wav', token: Optional[str] = None):  # 必须指定token获取任务
    token = token or await get_next_token_for_polling(FEISHU_URL)

    access_token = await get_access_token(token)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }

    async with httpx.AsyncClient(timeout=100) as client:
        # payload = {"extension": "wav"} # "{\"extension\":\"wav\"}"
        payload = "{\"extension\":\"wav\"}"
        payload = "{\"extension\":\"\"}"
        response = await client.post(f"{BASE_URL}/api/uploads/audio/", content=payload, headers=headers)

        # logger.debug(response.text)

        if response.is_success:
            data = response.json()
            logger.debug(data)

            file_id = data.get("id")
            # file_id = Path(data.get("fields").get('key')).stem

            payload = data.get("fields")
            files = {
                'file': file
                # 'file': ("xx", file, 'audio/mpeg')
                # 'file': (title, file, 'audio/wav')
            }

            response = await client.post(url=UPLOAD_BASE_UR, data=payload, files=files)
            logger.debug(response.status_code)
            if response.is_success:
                payload = {"upload_type": "file_upload", "upload_filename": title}
                response = await client.post(
                    f"{BASE_URL}/api/uploads/audio/{file_id}/upload-finish/",
                    headers=headers,
                    json=payload
                )
                for i in range(30):
                    response = await client.get(f"{BASE_URL}/api/uploads/audio/{file_id}/", headers=headers)
                    if response.is_success and response.json().get("s3_id"):
                        logger.debug(response.json())
                        break

                        # {
                        #     "id": "557f349e-2ce5-45f0-806c-efba18286599",
                        #     "status": "complete",
                        #     "error_message": null,
                        #     "s3_id": "m_476a2e21-a0aa-4e33-92ff-2b5ddd587661",
                        #     "title": "audio",
                        #     "image_url": "https://cdn1.suno.ai/image_476a2e21-a0aa-4e33-92ff-2b5ddd587661.png"
                        # }
                        # {"id": "5d39c704-52c2-4d32-9bef-ac06c2c84dab", "status": "error",
                        #  "error_message": "Uploaded audio is too short (currently 1.5 seconds). Minimum duration is 6 seconds.",
                        #  "s3_id": null, "title": "", "image_url": null}

                    elif response.json().get("status") == "error":
                        logger.error(response.text)
                        raise Exception(response.text)
                    if i == 1:
                        logger.debug(response.status_code)
                        logger.debug(response.text)
                        await asyncio.sleep(3)
                    else:
                        await asyncio.sleep(0.5)

                response = await client.post(
                    f"{BASE_URL}/api/uploads/audio/{file_id}/initialize-clip/",
                    headers=headers
                )
                clip_id = response.json().get("clip_id")  # m_{clip_id} image_{clip_id}.png

                payload = {"id": clip_id, "title": title}  # 好像不要也行，前端展示  title 传入文件名
                resp = await client.post(f"{BASE_URL}/api/gen/{clip_id}/set_metadata/", json=payload, headers=headers)

                data = await get_task(task_id=clip_id, token=token)  # 绑定token才能获取
                return data, token  # clip_id: token 存一下

            else:
                response.raise_for_status()


async def get_credits(token):
    access_token = await get_access_token(token)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=100, headers=headers) as client:
        response = await client.get(API_BILLING_INFO)
        response.raise_for_status()
        if response.is_success:
            data = response.json()
            # logger.debug(bjson(data))
            return data


async def check_token(token, threshold=10):
    try:
        data = await get_credits(token)

        logger.debug(data['total_credits_left'])

        return data['total_credits_left'] >= threshold  # 视频
    except Exception as e:
        logger.error(f"{e}\n无效\n{token}")
        return False


@retrying()
async def create_task_for_stems(stem_from_url):
    token = await get_next_token_for_polling(FEISHU_URL_STEM)
    access_token = await get_access_token(token)

    file = await to_bytes(stem_from_url)
    data, _ = await upload(file, token=token)
    logger.debug(bjson(data))
    stem_from_id = data[0]['id']  #

    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(f"/api/edit/stems/{stem_from_id}/")
        response.raise_for_status()
        if response.is_success:
            data = response.json()
            clip_ids = jsonpath.jsonpath(data, "$..id")

            task_id = f"suno-{','.join(clip_ids)}"  # 需要返回的任务id
            return Task(id=task_id, data=data, system_fingerprint=token)


@retrying()
async def create_task_for_cover(cover_from_url, lyrics):
    token = await get_next_token_for_polling(FEISHU_URL_STEM)
    access_token = await get_access_token(token)

    file = await to_bytes(cover_from_url)
    data, _ = await upload(file, token=token)
    cover_from_id = data[0]['id']

    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    payload = SunoAIRequest(
        task="cover",
        mv="chirp-v3-5-tau",
        cover_clip_id=cover_from_id,
        prompt=lyrics,
    ).model_dump(exclude_none=True)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(API_GENERATE_V2, json=payload)
        if response.is_success:
            data = response.json()
            task_id, *clip_ids = jsonpath.jsonpath(data, "$..id")
            clip_ids = [i for i in clip_ids if not str(i).startswith("m_")]

            task_id = f"suno-{','.join(clip_ids)}"  # 需要返回的任务id
            return Task(id=task_id, data=data, system_fingerprint=token)

        response.raise_for_status()


if __name__ == '__main__':
    # token = os.getenv("SUNO_API_KEY")
    # token = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8yaGlHSTlCZFVwOUdZcUlGM3ZmTU1IT25SNFAiLCJyb3RhdGluZ190b2tlbiI6ImltOWMzOGJ4bnV2OThiZXplMW8yOG1zd2Y2c3lrdzd6YnM2ejJubHkifQ.SnC8-G2LVQztTiA2davFS413mQIaBmRFDzIw1JmvHg4UOMXq95z0CgbfK8Gx8Zv-FXdpKVqkamiNTzZP9qsLOSgREqCSSq5bmA6SPIWx-R6dj1PMDFRX-qv5qGyyPe4sadF6wnr45MS9859148gRmr_Go8rAT_7Hu0DKySextl-Xbs6ClDaYYUyyV3HudWQh4F8jwvxkyer05AgN6smQH5eZI-NRKVgZn_i6Mtl8IJz8R1fzD2YNIcvH4QC4qGhrg9n74ljIeORCMsoJzW2SBZa4QWWDx_0VYs-tA_Z43bqwN_2ojMGM63fm2hLOZmwf6S1LQy9_O6UdcUQiEs__OA"
    token = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8ybEIwRFlXTWYya3NxNWlxazl4S2dZbmQ0eVciLCJyb3RhdGluZ190b2tlbiI6Im5vejdpMzlvY2YzMTEzaHp6aDZwY2N5MzltZm5xNmZvdnhtOG9oNngifQ.qUeBLl-NxKzhpoUBpo_EwkHH0qwIsG0nMSD_yvHv5EK7YfybupoPWw8SpSKDhmZu5K_KsgdOF0RQH22jll4U-x0BwfVu1ze-GBxjnNEoSerUB7hu1cfvmg7xMH8rHJQig2TWE2h0hzP6dMajPHQWRTltbb5MMkKHgBFj0CiAFqaGwzSSvAtERwwHBIK3KalbaV1oyd6DJYG4FrVgQLubkp7VXj11LszxD6qXklRhsc9h55kvYASDPHhnZJi9u2QfIbiKVkraXb6ShqDmNtqXbj22p6g2R9fwMEB-m68S7QSZyAWRArWzsSZujzhxmNuMGbuVX1v7op7F3hA2zFphYw"
    # print(arun(get_refresh_token(token)))
    print(arun(get_access_token(token)))

    # arun(generate_lyrics(prompt=''))

    # ids = "ee6d4369-3c75-4526-b6f1-b5f2f271cf30"
    # print(api_feed(api_key, ids))

    # for i in range(100):  # 测试过期时间
    #     print(api_billing_info(api_key))
    #     time.sleep(60)

    # print(arun(get_api_key()))
    # task_id = music_ids = 1
    # send_message(f"""
    #     https://api.chatfire.cn/task/suno/v1/tasks/{task_id}
    #     https://api.chatfire.cn/task/suno/v1/music/{music_ids}
    #     """)
    # file = open("/Users/betterme/PycharmProjects/AI/test.mp3", 'rb').read()
    # file = open("/Users/betterme/Downloads/audio.wav", 'rb').read()
    # arun(upload(file=file))
    # token = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8yajM1dkNXaTE1b0VQWnlUU0RMMnBsU3RiMVQiLCJyb3RhdGluZ190b2tlbiI6Ijd5MDV0cG1nMWJpbjBmaGIxZW1kYzZoazQzbDRuaG84bmJ4Yzc1dzgifQ.LsEfuPgwXu33f_UD2pRY4HjHIwU_rPG2rvG46BVDKqXcPmhRTWl5LjKgFrSzU51tgxfG-wMopVJhRxgS6YZUMtKVojDFtV_ZImyJ30u6LYA5nSbrkhqUrBdU4P5WmkL9irvh4sGPtvv8ML_pyXzsittsDrNnDCtm_isacOD-Fy3VKOCOjWj4W3qUUdnmBvTeeUbrnepQqurAjYPg6Ug-WkR43xZ5tWtxxTQ4ebtZglgRyCbyF3TM9XMKa67FTwHQsja8cLo2CyGRzb89e3uuF8Na_CY17ZlxJyQ2p_FZmL0egWr0EveZFeVzIUUs704pkhKd-RC9Q47Jqcg8qdiRYA"
    # arun(generate_lyrics('hi', token))
    # arun(generate_lyrics())
    # task_id = "0b017d4e-c559-4cc6-9339-9cd53aa25af4"
    # token = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8ybEIwRFlXTWYya3NxNWlxazl4S2dZbmQ0eVciLCJyb3RhdGluZ190b2tlbiI6Im5vejdpMzlvY2YzMTEzaHp6aDZwY2N5MzltZm5xNmZvdnhtOG9oNngifQ.qUeBLl-NxKzhpoUBpo_EwkHH0qwIsG0nMSD_yvHv5EK7YfybupoPWw8SpSKDhmZu5K_KsgdOF0RQH22jll4U-x0BwfVu1ze-GBxjnNEoSerUB7hu1cfvmg7xMH8rHJQig2TWE2h0hzP6dMajPHQWRTltbb5MMkKHgBFj0CiAFqaGwzSSvAtERwwHBIK3KalbaV1oyd6DJYG4FrVgQLubkp7VXj11LszxD6qXklRhsc9h55kvYASDPHhnZJi9u2QfIbiKVkraXb6ShqDmNtqXbj22p6g2R9fwMEB-m68S7QSZyAWRArWzsSZujzhxmNuMGbuVX1v7op7F3hA2zFphYw"
    # arun(get_task(task_id=task_id, token=token))

    # arun(get_credits(token))
    # arun(check_token(token))

    # arun(upload(Path('cover.mp3').read_bytes(), title="翻唱"))
    url = "https://cdn1.suno.ai/14d5afd8-057e-4d97-bc13-b8521dde568a.mp3"

    # arun(create_task_for_stems(url))
    #
    # arun(get_task("suno-644b4e39-abd5-44c0-95cb-25ee56b0f6ed,ee0443b2-af84-4fb5-b29d-d23d41ce8142", token=token))
    # lyrics = """
    # [Verse]\n每天天还未亮\n我们已经在路上\n辛苦工作拼命干\n工资却总是拖欠\n\n[Verse 2]\n加班加点不休息\n老板总是不提起\n口袋越来越空虚\n哪里是我们的权益\n\n[Chorus]\n一起团结呼吁\n要把话语权夺回\n为我们的权利\n我们绝不喊后退\n\n[Bridge]\n不怕辛劳与压力\n只怕公正被歪曲\n强忍泪水背负艰辛\n梦想不可被抹去\n\n[Verse 3]\n工资迟迟不见影\n梦想渐渐被耗尽\n但我们心不认命\n勇敢站出来喊停\n\n[Chorus]\n一起团结呼吁\n要把话语权夺回\n为我们的权利\n我们绝不喊后退
    # """
    # arun(create_task_for_cover(url, lyrics))

    # token = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8yazY3OG5pWWI5M2xjWHQ2a1dZMVNSalcxNVQiLCJyb3RhdGluZ190b2tlbiI6Imk2bzE0eXkxMW5mNTduaXFpcG9kOWx2cHBrbXBvbG16bTdqMDQ3bXQifQ.mD5dwo6c1xyg4kS_Z6Nwz4N_UonetHBFLcvHRTGTsqd4Pc-QYTEJ2IiedILX7nrA1b3K-mhN-M9isyzUxl3Pa9DCqQ4p9snsib_Rz-v_ou2V-rOORPHLwzIQSZbkE4gDyH4DF_UT6TqZRp_KkOyPEmyGQHQWjiUDuWv7CjDXM0AbqhV0gafCKA6cQ0ZdL9yKNq_0rFgIWP9mRrDGP6r237gXqLm84SmlhtMpob5T_QByCpunmTKe2Gi9qNEkKm_8EHAKSw3vj9XHBH3cMsiVPO47cT_rs98tIUq20sqk4C6hlL7mhLomm9L33k4wVTgyH7ln8UK9xxQ3lGm-W-mjKA"
    #
    # arun(get_credits(token))

    # arun(generate_lyrics('hi'))
    # arun(create_task(SunoAIRequest(prompt='写首中国风的歌曲')))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : background
# @Time         : 2024/6/12 11:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.decorators import background_task

from asgiref.sync import async_to_sync
from starlette.background import BackgroundTask as _BackgroundTask, BackgroundTasks as _BackgroundTasks


class BackgroundTask(_BackgroundTask):
    """
    async def f(x):
        for i in range(100):
            logger.debug(i)
            await asyncio.sleep(1)
    BackgroundTask(f, x='xxxxxxxxxx').call()

    bt = BackgroundTasks()
    bt.add_task(f, x='xxxxxxxxxx')
    bt.call()
    """

    @background_task
    def call(self):
        async_to_sync(super().__call__)()


class BackgroundTasks(_BackgroundTasks):
    """
    async def f(x):
        for i in range(100):
            logger.debug(i)
            await asyncio.sleep(1)
    BackgroundTask(f, x='xxxxxxxxxx').call()

    bt = BackgroundTasks()
    bt.add_task(f, x='xxxxxxxxxx')
    bt.call()
    """

    @background_task
    def call(self):
        async_to_sync(super().__call__)()


if __name__ == '__main__':
    from meutils.pipe import *


    async def f(x):
        logger.debug(x)

        for i in range(100):
            logger.debug(i)
            await asyncio.sleep(1)


    bt = BackgroundTasks()
    bt.add_task(f, x='xxxxxxxxxx')
    bt.call()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : send_email
# @Time         : 2020-03-04 13:58
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :

"""
# 小米邮件
# 机器自带邮件
# 邮件代理
"""

import smtplib
from email.mime.text import MIMEText
from email.header import Header

# ME
from meutils.pipe import *
from meutils.date_utils import date_difference


def send_email(subject="",
               msg: Union[str, pd.DataFrame] = "",
               receivers: Union[str, list] = 'meutils@qq.com',
               _subtype='html',
               msg_prefix='',
               msg_suffix='',
               msg_fn=lambda x: x,
               date=date_difference(fmt='%Y-%m-%d %H:%M:%S', days=0),
               host2sender=None,
               **kwargs):
    """

    :param subject: 主题
    :param msg:
    :param receivers:
    :param _subtype:
    :param msg_prefix:
    :param msg_suffix:
    :param msg_fn:
    :param kwargs:
    :return:
    """

    # init
    # token = get_zk_config("/push/email_token")
    # host, sender = list(token.items())[0]
    if host2sender is None:
        host2sender = {'localhost': 'BOT'}

    host, sender = list(host2sender.items())[0]
    smtp = smtplib.SMTP(host, 25)

    # 主题+内容
    subject = f"👉{subject}📅{date}"

    if isinstance(msg, pd.DataFrame):
        msg_fn = lambda df: df2bhtml(df, subject)
    msg = f"{msg_prefix}{msg_fn(msg)}{msg_suffix}"

    message = MIMEText(msg, _subtype, 'utf-8')
    message['Subject'] = Header(subject, 'utf-8')
    message['From'] = sender

    if isinstance(receivers, str) and receivers.__contains__("@"):
        receivers = [receivers]
    message['To'] = ",".join(receivers)

    try:
        smtp.sendmail(sender, receivers, message.as_string())
        logger.info("邮件发送成功")
    except smtplib.SMTPException as e:
        logger.warning(f"{e}: 无法发送邮件")


def nesc_email():
    import yagmail
    yagmail_server = yagmail.SMTP(
        user="xx@xx.cn",
        password="Qq-xx",
        host="mail.xx.cn"
    )

    contents = ['Hi, baby.', 'One', 'Two']
    yagmail_server.send('xx@xx.cn', '测试', contents)


if __name__ == '__main__':
    send_email("测试邮件", msg='邮件内容')
    send_email("测试邮件", msg=pd.DataFrame(np.random.random((5, 5))))



import json
import traceback

from fastapi import status
from starlette.requests import Request
from starlette.responses import JSONResponse

from fastapi.exceptions import RequestValidationError, HTTPException

from meutils.notice.feishu import send_message

UNAUTHORIZED_HTTPException = HTTPException(
    status_code=status.HTTP_401_UNAUTHORIZED,
    detail={
        "error": {
            "message": "",
            "type": "invalid_request_error",
            "param": None,
            "code": "invalid_api_key",
        }
    })


async def http_error_handler(_: Request, exc: HTTPException) -> JSONResponse:
    # print(exc)
    content = {
        "error":
            {
                "message": f"{exc.detail}",
                "type": "http-error",
            }
    }
    return JSONResponse(
        content=content,
        status_code=exc.status_code
    )


async def general_exception_handler(request: Request, exc: Exception):
    return JSONResponse(
        content={"message": str(exc)},
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


async def chatfire_api_exception_handler(request: Request, exc: Exception):
    content = {
        "error":
            {
                "message": f"{exc}",
                "type": "cf-api-error",
            },

        # "code": status.HTTP_500_INTERNAL_SERVER_ERROR
    }
    content_detail = f"{traceback.format_exc()}"

    send_message(
        content_detail,
        title="cf-api-error",
        url="https://open.feishu.cn/open-apis/bot/v2/hook/79fc258f-46a9-419e-b131-1d79b3d0bcff"
    )

    # send_message
    return JSONResponse(
        content=content,
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


if __name__ == '__main__':
    from meutils.notice.feishu import send_message



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : crontab
# @Time         : 2021/1/24 12:04 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://gitlab.com/doctormo/python-crontab/


import os
import re
import shlex

import types
import codecs
import logging
import tempfile
import platform
import subprocess as sp

from time import sleep
from datetime import time, date, datetime, timedelta
from loguru import logger

try:
    from collections import OrderedDict
except ImportError:
    # python 2.6 and below causes this error
    try:
        from ordereddict import OrderedDict
    except ImportError:
        raise ImportError("OrderedDict is required for python-crontab, you can"
                          " install ordereddict 1.1 from pypi for python2.6")

__pkgname__ = 'python-crontab'
__version__ = '2.5.1'

ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                     r'\s+([^@#\s]+)\s+([^\n]*?)(\s+#\s*([^\n]*)|$)')
SPECREX = re.compile(r'^\s*@(\w+)\s([^#\n]*)(\s+#\s*([^\n]*)|$)')
DEVNULL = ">/dev/null 2>&1"

WEEK_ENUM = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun']

MONTH_ENUM = [None, 'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug',
              'sep', 'oct', 'nov', 'dec']

SPECIALS = {"reboot": '@reboot',
            "hourly": '0 * * * *',
            "daily": '0 0 * * *',
            "weekly": '0 0 * * 0',
            "monthly": '0 0 1 * *',
            "yearly": '0 0 1 1 *',
            "annually": '0 0 1 1 *',
            "midnight": '0 0 * * *'}

SPECIAL_IGNORE = ['midnight', 'annually']

S_INFO = [
    {'max': 59, 'min': 0, 'name': 'Minutes'},
    {'max': 23, 'min': 0, 'name': 'Hours'},
    {'max': 31, 'min': 1, 'name': 'Day of Month'},
    {'max': 12, 'min': 1, 'name': 'Month', 'enum': MONTH_ENUM},
    {'max': 6, 'min': 0, 'name': 'Day of Week', 'enum': WEEK_ENUM},
]

# Detect Python3 and which OS for temperments.
PY3 = platform.python_version()[0] == '3'
WINOS = platform.system() == 'Windows'
POSIX = os.name == 'posix'
SYSTEMV = not WINOS and os.uname()[0] in ["SunOS", "AIX", "HP-UX"]
SYSTEMV = not WINOS and (
        os.uname()[0] in ["SunOS", "AIX", "HP-UX"]
        or
        os.uname()[4] in ["mips"]
)

# Switch this on if you want your crontabs to have zero padding.
ZERO_PAD = False

# LOG = logging.getLogger('crontab')
LOG = logger

CRONCMD = "/usr/bin/crontab"
SHELL = os.environ.get('SHELL', '/bin/sh')
# The shell won't actually work on windows here, but
# it should be updated later in the below conditional.

# pylint: disable=W0622,invalid-name,too-many-public-methods
# pylint: disable=function-redefined,too-many-instance-attributes
current_user = lambda: None
if not WINOS:
    import pwd


    def current_user():
        """Returns the username of the current user"""
        return pwd.getpwuid(os.getuid())[0]

if PY3:
    unicode = str
    basestring = str


def open_pipe(cmd, *args, **flags):
    """Runs a program and orders the arguments for compatability.

    a. keyword args are flags and always appear /before/ arguments for bsd
    """
    cmd_args = tuple(shlex.split(cmd, posix=flags.pop('posix', POSIX)))
    env = flags.pop('env', None)
    for (key, value) in flags.items():
        if len(key) == 1:
            cmd_args += (("-%s" % key),)
            if value is not None:
                cmd_args += (unicode(value),)
        else:
            cmd_args += (("--%s=%s" % (key, value)),)
    args = tuple(arg for arg in (cmd_args + tuple(args)) if arg)
    return sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE, env=env)


def _unicode(text):
    """Convert to the best string format for this python version"""
    if isinstance(text, str) and not PY3:
        return unicode(text, 'utf-8')
    if isinstance(text, bytes) and PY3:
        return text.decode('utf-8')
    return text


class CronTab(object):
    """可配置每天第一分钟同步crontab配置信息（zk/yaml） 1 0 * * *

    Crontab object which can access any time based cron using the standard.

    user    - Set the user of the crontab (default: None)
      * 'user' = Load from $username's crontab (instead of tab or tabfile)
      * None   = Don't load anything from any user crontab.
      * True   = Load from current $USER's crontab (unix only)
      * False  = This is a system crontab, each command has a username

    tab     - Use a string variable as the crontab instead of installed crontab
    tabfile - Use a file for the crontab instead of installed crontab
    log     - Filename for logfile instead of /var/log/syslog

    """

    def __init__(self, user=None, tab=None, tabfile=None, log=None):
        self.lines = None
        self.crons = None
        self.filen = None
        self.env = None
        self._parked_env = OrderedDict()
        # Protect windows users
        self.root = not WINOS and os.getuid() == 0
        # Storing user flag / username
        self._user = user
        # Load string or filename as inital crontab
        self.intab = tab
        self.read(tabfile)
        self._log = log

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.write()

    @property
    def log(self):
        """Returns the CronLog object for this tab (user or root tab only)"""
        from cronlog import CronLog
        if self._log is None or isinstance(self._log, basestring):
            self._log = CronLog(self._log, user=self.user or 'root')
        return self._log

    @property
    def user(self):
        """Return user's username of this crontab if applicable"""
        if self._user is True:
            return current_user()
        return self._user

    @property
    def user_opt(self):
        """Returns the user option for the crontab commandline"""
        # Fedora and Mac require the current user to not specify
        # But Ubuntu/Debian doesn't care. Be careful here.
        if self._user and self._user is not True:
            if self._user != current_user():
                return {'u': self._user}
        return {}

    def __setattr__(self, name, value):
        """Catch setting crons and lines directly"""
        if name == 'lines' and value:
            for line in value:
                self.append(CronItem.from_line(line, cron=self), line, read=True)
        elif name == 'crons' and value:
            raise AttributeError("You can NOT set crons attribute directly")
        else:
            super(CronTab, self).__setattr__(name, value)

    def read(self, filename=None):
        """
        Read in the crontab from the system into the object, called
        automatically when listing or using the object. use for refresh.
        """
        self.crons = []
        self.lines = []
        self.env = OrderedVariableList()
        lines = []

        if self.intab is not None:
            lines = self.intab.split('\n')

        elif filename:
            self.filen = filename
            with codecs.open(filename, 'r', encoding='utf-8') as fhl:
                lines = fhl.readlines()

        elif self.user:
            (out, err) = open_pipe(CRONCMD, l='', **self.user_opt).communicate()
            if err and 'no crontab for' in unicode(err):
                pass
            elif err:
                raise IOError("Read crontab %s: %s" % (self.user, err))
            lines = out.decode('utf-8').split("\n")

        self.lines = lines

    def append(self, item, line='', read=False):
        """Append a CronItem object to this CronTab"""
        if item.is_valid():
            item.env.update(self._parked_env)
            self._parked_env = OrderedDict()
            if read and not item.comment and self.lines and \
                    self.lines[-1] and self.lines[-1][0] == '#':
                item.set_comment(self.lines.pop()[1:].strip(), True)

            self.crons.append(item)
            self.lines.append(item)
        elif '=' in line:
            if ' ' not in line or line.index('=') < line.index(' '):
                (name, value) = line.split('=', 1)
                value = value.strip()
                for quot in "\"'":
                    if value[0] == quot and value[-1] == quot:
                        value = value.strip(quot)
                        break
                self._parked_env[name.strip()] = value
        else:
            if not self.crons and self._parked_env:
                self.env.update(self._parked_env)
                self._parked_env = OrderedDict()
            self.lines.append(line.replace('\n', ''))

    def write(self, filename=None, user=None, errors=False):
        """Write the crontab to it's source or a given filename."""
        if filename:
            self.filen = filename
        elif user is not None:
            self.filen = None
            self.intab = None
            self._user = user

        # Add to either the crontab or the internal tab.
        if self.intab is not None:
            self.intab = self.render()
            # And that's it if we never saved to a file
            if not self.filen:
                return

        if self.filen:
            fileh = open(self.filen, 'wb')
        else:
            filed, path = tempfile.mkstemp()
            fileh = os.fdopen(filed, 'wb')

        fileh.write(self.render(errors=errors).encode('utf-8'))
        fileh.close()

        if not self.filen:
            # Add the entire crontab back to the user crontab
            if not self.user:
                os.unlink(path)
                raise IOError("Please specify user or filename to write.")

            proc = open_pipe(CRONCMD, path, **self.user_opt)
            ret = proc.wait()
            if ret != 0:
                raise IOError("Program Error: {} returned {}: {}".format(
                    CRONCMD, ret, proc.stderr.read()))
            proc.stdout.close()
            proc.stderr.close()
            os.unlink(path)

    def write_to_user(self, user=True):
        """Write the crontab to a user (or root) instead of a file."""
        return self.write(user=user)

    def run_pending(self, **kwargs):
        """Run all commands in this crontab if pending (generator)"""
        for job in self:
            ret = job.run_pending(**kwargs)
            if ret not in [None, -1]:
                yield ret

    def run_scheduler(self, timeout=-1, **kwargs):
        """Run the CronTab as an internal scheduler (generator)"""
        count = 0
        while count != timeout:
            now = datetime.now()
            if 'warp' in kwargs:
                now += timedelta(seconds=count * 60)
            for value in self.run_pending(now=now):
                yield value

            sleep(kwargs.get('cadence', 60))
            count += 1

    def render(self, errors=False):
        """Render this crontab as it would be in the crontab.

        errors - Should we not comment out invalid entries and cause errors?
        """
        crons = []
        for line in self.lines:
            if isinstance(line, (unicode, str)):
                if line.strip().startswith('#') or not line.strip():
                    crons.append(line.strip())
                elif not errors:
                    crons.append('# DISABLED LINE\n# ' + line)
                else:
                    raise ValueError("Invalid line: %s" % line)
            elif isinstance(line, CronItem):
                if not line.is_valid() and not errors:
                    line.enabled = False
                crons.append(unicode(line).strip())

        # Environment variables are attached to cron lines so order will
        # always work no matter how you add lines in the middle of the stack.
        result = unicode(self.env) + u'\n'.join(crons)
        if result and result[-1] not in (u'\n', u'\r'):
            result += u'\n'
        return result

    def new(self, command='', comment='', user=None, pre_comment=False):
        """
        Create a new cron with a command and comment.

        Returns the new CronItem object.
        """
        if not user and self.user is False:
            raise ValueError("User is required for system crontabs.")
        item = CronItem(command, comment, user=user, cron=self, pre_comment=pre_comment)
        self.append(item)
        return item

    def find_command(self, command):
        """Return an iter of jobs matching any part of the command."""
        for job in list(self.crons):
            if isinstance(command, type(ITEMREX)):
                if command.findall(job.command):
                    yield job
            elif command in job.command:
                yield job

    def find_comment(self, comment):
        """Return an iter of jobs that match the comment field exactly."""
        for job in list(self.crons):
            if isinstance(comment, type(ITEMREX)):
                if comment.findall(job.comment):
                    yield job
            elif comment == job.comment:
                yield job

    # todo: 支持正则
    def find_comment_prefix(self, comment):
        """Return an iter of jobs that match the comment field exactly."""
        for job in list(self.crons):
            if job.comment.startswith(comment):
                yield job

    def find_time(self, *args):
        """Return an iter of jobs that match this time pattern"""
        for job in list(self.crons):
            if job.slices == CronSlices(*args):
                yield job

    @property
    def commands(self):
        """Return a generator of all unqiue commands used in this crontab"""
        returned = []
        for cron in self.crons:
            if cron.command not in returned:
                yield cron.command
                returned.append(cron.command)

    @property
    def comments(self):
        """Return a generator of all unique comments/Id used in this crontab"""
        returned = []
        for cron in self.crons:
            if cron.comment and cron.comment not in returned:
                yield cron.comment
                returned.append(cron.comment)

    def remove_all(self, *args, **kwargs):
        """Removes all crons using the stated command OR that have the
        stated comment OR removes everything if no arguments specified.

           command - Remove all with this command
           comment - Remove all with this comment or ID
           time    - Remove all with this time code
        """
        if args:
            raise AttributeError("Invalid use: remove_all(command='cmd')")
        if 'command' in kwargs:
            return self.remove(*self.find_command(kwargs['command']))
        elif 'comment' in kwargs:
            return self.remove(*self.find_comment(kwargs['comment']))
        elif 'time' in kwargs:
            return self.remove(*self.find_time(kwargs['time']))
        return self.remove(*self.crons[:])

    def remove(self, *items):
        """Remove a selected cron from the crontab."""
        result = 0
        for item in items:
            if isinstance(item, (list, tuple, types.GeneratorType)):
                for subitem in item:
                    result += self._remove(subitem)
            elif isinstance(item, CronItem):
                result += self._remove(item)
            else:
                raise TypeError("You may only remove CronItem objects, " \
                                "please use remove_all() to specify by name, id, etc.")
        return result

    def _remove(self, item):
        """Internal removal of an item"""
        # Manage siblings when items are deleted
        for sibling in self.lines[self.lines.index(item) + 1:]:
            if isinstance(sibling, CronItem):
                env = sibling.env
                sibling.env = item.env
                sibling.env.update(env)
                sibling.env.job = sibling
                break
            elif sibling == '':
                self.lines.remove(sibling)
            else:
                break

        self.crons.remove(item)
        self.lines.remove(item)
        return 1

    def __repr__(self):
        kind = 'System ' if self._user is False else ''
        if self.filen:
            return "<%sCronTab '%s'>" % (kind, self.filen)
        elif self.user and not self.user_opt:
            return "<My CronTab>"
        elif self.user:
            return "<User CronTab '%s'>" % self.user
        return "<Unattached %sCronTab>" % kind

    def __iter__(self):
        """Return generator so we can track jobs after removal"""
        for job in list(self.crons.__iter__()):
            yield job

    def __getitem__(self, i):
        return self.crons[i]

    def __unicode__(self):
        return self.render()

    def __len__(self):
        return len(self.crons)

    def __str__(self):
        return self.render()


class CronItem(object):
    """
    An item which objectifies a single line of a crontab and
    May be considered to be a cron job object.
    """

    def __init__(self, command='', comment='', user=None, cron=None, pre_comment=False):
        self.cron = cron
        self.user = user
        self.valid = False
        self.enabled = True
        self.special = False
        self.comment = None
        self.command = None
        self.last_run = None
        self.env = OrderedVariableList(job=self)

        # Marker labels Ansible jobs etc
        self.pre_comment = False
        self.marker = None
        self.stdin = None
        self._log = None

        # Initalise five cron slices using static info.
        self.slices = CronSlices()

        self.set_comment(comment, pre_comment)

        if command:
            self.set_command(command)

    def __hash__(self):
        return hash((self.command, self.comment, self.hour, self.minute, self.dow))

    def __eq__(self, other):
        if not isinstance(other, CronItem):
            return False
        return self.__hash__() == other.__hash__()

    @classmethod
    def from_line(cls, line, user=None, cron=None):
        """Generate CronItem from a cron-line and parse out command and comment"""
        obj = cls(user=user, cron=cron)
        obj.parse(line.strip())
        return obj

    def delete(self):
        """Delete this item and remove it from it's parent"""
        if not self.cron:
            raise UnboundLocalError("Cron item is not in a crontab!")
        else:
            self.cron.remove(self)

    def set_command(self, cmd, parse_stdin=False):
        """Set the command and filter as needed"""
        if parse_stdin:
            cmd = cmd.replace('%', '\n').replace('\\\n', '%')
            if '\n' in cmd:
                cmd, self.stdin = cmd.split('\n', 1)
        self.command = _unicode(cmd.strip())
        self.valid = True

    def set_comment(self, cmt, pre_comment=False):
        """Set the comment and don't filter, pre_comment indicates comment appears
        before the cron, otherwise it appears ont he same line after the command.
        """
        if cmt and cmt[:8] == 'Ansible:':
            self.marker = 'Ansible'
            cmt = cmt[8:].lstrip()
            pre_comment = True

        self.comment = cmt
        self.pre_comment = pre_comment

    def parse(self, line):
        """Parse a cron line string and save the info as the objects."""
        line = _unicode(line)
        if not line or line[0] == '#':
            self.enabled = False
            line = line[1:].strip()
        # We parse all lines so we can detect disabled entries.
        self._set_parse(ITEMREX.findall(line))
        self._set_parse(SPECREX.findall(line))

    def _set_parse(self, result):
        """Set all the parsed variables into the item"""
        if not result:
            return
        self.comment = result[0][-1]
        if self.cron.user is False:
            # Special flag to look for per-command user
            ret = result[0][-3].split(None, 1)
            self.set_command(ret[-1], True)
            if len(ret) == 2:
                self.user = ret[0]
            else:
                self.valid = False
                self.enabled = False
                LOG.error(str("Missing user or command in system cron line."))
        else:
            self.set_command(result[0][-3], True)
        try:
            self.setall(*result[0][:-3])
        except (ValueError, KeyError) as err:
            if self.enabled:
                LOG.error(str(err))
            self.valid = False
            self.enabled = False

    def enable(self, enabled=True):
        """Set if this cron job is enabled or not"""
        if enabled in [True, False]:
            self.enabled = enabled
        return self.enabled

    def is_enabled(self):
        """Return true if this job is enabled (not commented out)"""
        return self.enabled

    def is_valid(self):
        """Return true if this job is valid"""
        return self.valid

    def render(self):
        """Render this set cron-job to a string"""
        command = _unicode(self.command).replace(u'%', u'\\%')
        user = ''
        if self.cron and self.cron.user is False:
            if not self.user:
                raise ValueError("Job to system-cron format, no user set!")
            user = self.user + ' '
        result = u"%s %s%s" % (unicode(self.slices), user, command)
        if self.stdin:
            result += ' %' + self.stdin.replace('\n', '%')
        if not self.enabled:
            result = u"# " + result
        if self.comment:
            comment = self.comment = _unicode(self.comment)
            if self.marker:
                comment = u"#%s: %s" % (self.marker, comment)
            else:
                comment = u"# " + comment

            if SYSTEMV or self.pre_comment or self.stdin:
                result = comment + "\n" + result
            else:
                result += ' ' + comment

        return unicode(self.env) + result

    def every_reboot(self):
        """Set to every reboot instead of a time pattern: @reboot"""
        self.clear()
        return self.slices.setall('@reboot')

    def every(self, unit=1):
        """
        Replace existing time pattern with a single unit, setting all lower
        units to first value in valid range.

        For instance job.every(3).days() will be `0 0 */3 * *`
        while job.day().every(3) would be `* * */3 * *`

        Many of these patterns exist as special tokens on Linux, such as
        `@midnight` and `@hourly`
        """
        return Every(self.slices, unit)

    def setall(self, *args):
        """Replace existing time pattern with these five values given as args:

           job.setall("1 2 * * *")
           job.setall(1, 2) == '1 2 * * *'
           job.setall(0, 0, None, '>', 'SUN') == '0 0 * 12 SUN'
        """
        return self.slices.setall(*args)

    def clear(self):
        """Clear the special and set values"""
        return self.slices.clear()

    def frequency(self, year=None):
        """Returns the number of times this item will execute in a given year
           (defaults to this year)
        """
        return self.slices.frequency(year=year)

    def frequency_per_year(self, year=None):
        """Returns the number of /days/ this item will execute on in a year
           (defaults to this year)
        """
        return self.slices.frequency_per_year(year=year)

    def frequency_per_day(self):
        """Returns the number of time this item will execute in any day"""
        return self.slices.frequency_per_day()

    def frequency_per_hour(self):
        """Returns the number of times this item will execute in any hour"""
        return self.slices.frequency_per_hour()

    def run_pending(self, now=None):
        """Runs the command if scheduled"""
        now = now or datetime.now()
        if self.is_enabled():
            if self.last_run is None:
                self.last_run = now

            next_time = self.schedule(self.last_run).get_next()
            if next_time < now:
                self.last_run = now
                return self.run()
        return -1

    def run(self):
        """Runs the given command as a pipe"""
        env = os.environ.copy()
        env.update(self.env.all())
        shell = self.env.get('SHELL', SHELL)
        (out, err) = open_pipe(shell, '-c', self.command, env=env).communicate()
        if err:
            LOG.error(err.decode("utf-8"))
        return out.decode("utf-8").strip()

    def schedule(self, date_from=None):
        """Return a croniter schedule if available."""
        if not date_from:
            date_from = datetime.now()
        try:
            # Croniter is an optional import
            from croniter.croniter import croniter
        except ImportError:
            raise ImportError("Croniter not available. Please install croniter"
                              " python module via pip or your package manager")
        return croniter(self.slices.clean_render(), date_from, ret_type=datetime)

    def description(self, **kw):
        """
        Returns a description of the crontab's schedule (if available)

        **kw - Keyword arguments to pass to cron_descriptor (see docs)
        """
        try:
            from cron_descriptor import ExpressionDescriptor
        except ImportError:
            raise ImportError("cron_descriptor not available. Please install" \
                              "cron_descriptor python module via pip or your package manager")

        exdesc = ExpressionDescriptor(self.slices.clean_render(), **kw)
        return exdesc.get_description()

    @property
    def log(self):
        """Return a cron log specific for this job only"""
        if not self._log and self.cron:
            self._log = self.cron.log.for_program(self.command)
        return self._log

    @property
    def minute(self):
        """Return the minute slice"""
        return self.slices[0]

    @property
    def minutes(self):
        """Same as minute"""
        return self.minute

    @property
    def hour(self):
        """Return the hour slice"""
        return self.slices[1]

    @property
    def hours(self):
        """Same as hour"""
        return self.hour

    @property
    def day(self):
        """Return the day slice"""
        return self.dom

    @property
    def dom(self):
        """Return the day-of-the month slice"""
        return self.slices[2]

    @property
    def month(self):
        """Return the month slice"""
        return self.slices[3]

    @property
    def months(self):
        """Same as month"""
        return self.month

    @property
    def dow(self):
        """Return the day of the week slice"""
        return self.slices[4]

    def __repr__(self):
        return "<CronItem '%s'>" % unicode(self)

    def __len__(self):
        return len(unicode(self))

    def __getitem__(self, key):
        return self.slices[key]

    def __lt__(self, value):
        return self.frequency() < CronSlices(value).frequency()

    def __gt__(self, value):
        return self.frequency() > CronSlices(value).frequency()

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        if not self.is_valid() and self.enabled:
            raise ValueError('Refusing to render invalid crontab.'
                             ' Disable to continue.')
        return self.render()


class Every(object):
    """Provide an interface to the job.every() method:
        Available Calls:
          minute, minutes, hour, hours, dom, doms, month, months, dow, dows

       Once run all units will be cleared (set to *) then proceeding units
       will be set to '0' and the target unit will be set as every x units.
    """

    def __init__(self, item, units):
        self.slices = item
        self.unit = units
        for (key, name) in enumerate(['minute', 'hour', 'dom', 'month', 'dow',
                                      'min', 'hour', 'day', 'moon', 'weekday']):
            setattr(self, name, self.set_attr(key % 5))
            setattr(self, name + 's', self.set_attr(key % 5))

    def set_attr(self, target):
        """Inner set target, returns function"""

        def innercall():
            """Returned inner call for setting slice targets"""
            self.slices.clear()
            # Day-of-week is actually a level 2 set, not level 4.
            for key in range(target == 4 and 2 or target):
                self.slices[key].on('<')
            self.slices[target].every(self.unit)

        return innercall

    def year(self):
        """Special every year target"""
        if self.unit > 1:
            raise ValueError("Invalid value '%s', outside 1 year" % self.unit)
        self.slices.setall('@yearly')


class CronSlices(list):
    """Controls a list of five time 'slices' which reprisent:
        minute frequency, hour frequency, day of month frequency,
        month requency and finally day of the week frequency.
     """

    def __init__(self, *args):
        super(CronSlices, self).__init__([CronSlice(info) for info in S_INFO])
        self.special = None
        self.setall(*args)
        self.is_valid = self.is_self_valid

    def is_self_valid(self, *args):
        """Object version of is_valid"""
        return CronSlices.is_valid(*(args or (self,)))

    @classmethod
    def is_valid(cls, *args):  # pylint: disable=method-hidden
        """Returns true if the arguments are valid cron pattern"""
        try:
            return bool(cls(*args))
        except (ValueError, KeyError):
            return False

    def setall(self, *slices):
        """Parses the various ways date/time frequency can be specified"""
        self.clear()
        if len(slices) == 1:
            (slices, self.special) = self._parse_value(slices[0])
            if slices[0] == '@reboot':
                return
        if id(slices) == id(self):
            raise AssertionError("Can not set cron to itself!")
        for set_a, set_b in zip(self, slices):
            set_a.parse(set_b)

    @staticmethod
    def _parse_value(value):
        """Parse a single value into an array of slices"""
        if isinstance(value, basestring) and value:
            return CronSlices._parse_str(value)
        if isinstance(value, CronItem):
            return value.slices, None
        elif isinstance(value, datetime):
            return [value.minute, value.hour, value.day, value.month, '*'], None
        elif isinstance(value, time):
            return [value.minute, value.hour, '*', '*', '*'], None
        elif isinstance(value, date):
            return [0, 0, value.day, value.month, '*'], None
            # It might be possible to later understand timedelta objects
            # but there's no convincing mathematics to do the conversion yet.
        elif not isinstance(value, (list, tuple)):
            raise ValueError("Unknown type: {}".format(type(value).__name__))
        return value, None

    @staticmethod
    def _parse_str(value):
        """Parse a string which contains slice information"""
        key = value.lstrip('@').lower()
        if value.count(' ') == 4:
            return value.strip().split(' '), None
        elif key in SPECIALS.keys():
            return SPECIALS[key].split(' '), '@' + key
        elif value.startswith('@'):
            raise ValueError("Unknown special '{}'".format(value))
        return [value], None

    def clean_render(self):
        """Return just numbered parts of this crontab"""
        return ' '.join([unicode(s) for s in self])

    def render(self):
        "Return just the first part of a cron job (the numbers or special)"
        slices = self.clean_render()
        if self.special:
            if self.special == '@reboot' or \
                    SPECIALS[self.special.strip('@')] == slices:
                return self.special
        if not SYSTEMV:
            for (name, value) in SPECIALS.items():
                if value == slices and name not in SPECIAL_IGNORE:
                    return "@%s" % name
        return slices

    def clear(self):
        """Clear the special and set values"""
        self.special = None
        for item in self:
            item.clear()

    def frequency(self, year=None):
        """Return frequence per year times frequency per day"""
        return self.frequency_per_year(year=year) * self.frequency_per_day()

    def frequency_per_year(self, year=None):
        """Returns the number of times this item will execute
           in a given year (default is this year)"""
        result = 0
        if not year:
            year = date.today().year

        weekdays = list(self[4])

        for month in self[3]:
            for day in self[2]:
                try:
                    if date(year, month, day).weekday() in weekdays:
                        result += 1
                except ValueError:
                    continue
        return result

    def frequency_per_day(self):
        """Returns the number of times this item will execute in any day"""
        return len(self[0]) * len(self[1])

    def frequency_per_hour(self):
        """Returns the number of times this item will execute in any hour"""
        return len(self[0])

    def __str__(self):
        return self.render()

    def __eq__(self, arg):
        return self.render() == CronSlices(arg).render()


class SundayError(KeyError):
    """Sunday was specified as 7 instead of 0"""
    pass


class Also(object):
    """Link range values together (appending instead of replacing)"""

    def __init__(self, obj):
        self.obj = obj

    def every(self, *a):
        """Also every one of these"""
        return self.obj.every(*a, also=True)

    def on(self, *a):
        """Also on these"""
        return self.obj.on(*a, also=True)

    def during(self, *a):
        """Also during these"""
        return self.obj.during(*a, also=True)


class CronSlice(object):
    """Cron slice object which shows a time pattern"""

    def __init__(self, info, value=None):
        if isinstance(info, int):
            info = S_INFO[info]
        self.min = info.get('min', None)
        self.max = info.get('max', None)
        self.name = info.get('name', None)
        self.enum = info.get('enum', None)
        self.parts = []
        if value:
            self.parse(value)

    def __hash__(self):
        return hash(str(self))

    def parse(self, value):
        """Set values into the slice."""
        self.clear()
        if value is not None:
            for part in unicode(value).split(','):
                if part.find("/") > 0 or part.find("-") > 0 or part == '*':
                    self.parts += self.get_range(part)
                    continue
                self.parts.append(self.parse_value(part, sunday=0))

    def render(self, resolve=False):
        """Return the slice rendered as a crontab.

        resolve - return integer values instead of enums (default False)

        """
        if not self.parts:
            return '*'
        return _render_values(self.parts, ',', resolve)

    def __repr__(self):
        return "<CronSlice '%s'>" % unicode(self)

    def __eq__(self, value):
        return unicode(self) == unicode(value)

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return self.render()

    def every(self, n_value, also=False):
        """Set the every X units value"""
        if not also:
            self.clear()
        self.parts += self.get_range(int(n_value))
        return self.parts[-1]

    def on(self, *n_value, **opts):
        """Set the time values to the specified placements."""
        if not opts.get('also', False):
            self.clear()
        for set_a in n_value:
            self.parts += (self.parse_value(set_a, sunday=0),)
        return self.parts

    def during(self, vfrom, vto, also=False):
        """Set the During value, which sets a range"""
        if not also:
            self.clear()
        self.parts += self.get_range(unicode(vfrom) + '-' + unicode(vto))
        return self.parts[-1]

    @property
    def also(self):
        """Appends rather than replaces the new values"""
        return Also(self)

    def clear(self):
        """clear the slice ready for new vaues"""
        self.parts = []

    def get_range(self, *vrange):
        """Return a cron range for this slice"""
        ret = CronRange(self, *vrange)
        if ret.dangling is not None:
            return [ret.dangling, ret]
        return [ret]

    def __iter__(self):
        """Return the entire element as an iterable"""
        ret = {}
        # An empty part means '*' which is every(1)
        if not self.parts:
            self.every(1)
        for part in self.parts:
            if isinstance(part, CronRange):
                for bit in part.range():
                    ret[bit] = 1
            else:
                ret[int(part)] = 1
        for val in ret:
            yield val

    def __len__(self):
        """Returns the number of times this slice happens in it's range"""
        return len(list(self.__iter__()))

    def parse_value(self, val, sunday=None):
        """Parse the value of the cron slice and raise any errors needed"""
        if val == '>':
            val = self.max
        elif val == '<':
            val = self.min
        try:
            out = get_cronvalue(val, self.enum)
        except ValueError:
            raise ValueError("Unrecognised %s: '%s'" % (self.name, val))
        except KeyError:
            raise KeyError("No enumeration for %s: '%s'" % (self.name, val))

        if self.max == 6 and int(out) == 7:
            if sunday is not None:
                return sunday
            raise SundayError("Detected Sunday as 7 instead of 0!")

        if int(out) < self.min or int(out) > self.max:
            raise ValueError("'{1}', not in {0.min}-{0.max} for {0.name}".format(self, val))
        return out


def get_cronvalue(value, enums):
    """Returns a value as int (pass-through) or a special enum value"""
    if isinstance(value, int):
        return value
    elif unicode(value).isdigit():
        return int(str(value))
    if not enums:
        raise KeyError("No enumeration allowed")
    return CronValue(unicode(value), enums)


class CronValue(object):  # pylint: disable=too-few-public-methods
    """Represent a special value in the cron line"""

    def __init__(self, value, enums):
        self.text = value
        self.value = enums.index(value.lower())

    def __lt__(self, value):
        return self.value < int(value)

    def __repr__(self):
        return unicode(self)

    def __str__(self):
        return self.text

    def __int__(self):
        return self.value


def _render_values(values, sep=',', resolve=False):
    """Returns a rendered list, sorted and optionally resolved"""
    if len(values) > 1:
        values.sort()
    return sep.join([_render(val, resolve) for val in values])


def _render(value, resolve=False):
    """Return a single value rendered"""
    if isinstance(value, CronRange):
        return value.render(resolve)
    if resolve:
        return str(int(value))
    return unicode(u'{:02d}'.format(value) if ZERO_PAD else value)


class CronRange(object):
    """A range between one value and another for a time range."""

    def __init__(self, vslice, *vrange):
        # holds an extra dangling entry, for example sundays.
        self.dangling = None
        self.slice = vslice
        self.cron = None
        self.seq = 1

        if not vrange:
            self.all()
        elif isinstance(vrange[0], basestring):
            self.parse(vrange[0])
        elif isinstance(vrange[0], (int, CronValue)):
            if len(vrange) == 2:
                (self.vfrom, self.vto) = vrange
            else:
                self.seq = vrange[0]
                self.all()

    def parse(self, value):
        """Parse a ranged value in a cronjob"""
        if value.count('/') == 1:
            value, seq = value.split('/')
            try:
                self.seq = self.slice.parse_value(seq)
            except SundayError:
                self.seq = 1
                value = "0-0"
            if self.seq < 1 or self.seq > self.slice.max:
                raise ValueError("Sequence can not be divided by zero or max")
        if value.count('-') == 1:
            vfrom, vto = value.split('-')
            self.vfrom = self.slice.parse_value(vfrom, sunday=0)
            try:
                self.vto = self.slice.parse_value(vto)
            except SundayError:
                if self.vfrom == 1:
                    self.vfrom = 0
                else:
                    self.dangling = 0
                self.vto = self.slice.parse_value(vto, sunday=6)
            if self.vto < self.vfrom:
                raise ValueError("Bad range '{0.vfrom}-{0.vto}'".format(self))
        elif value == '*':
            self.all()
        else:
            raise ValueError('Unknown cron range value "%s"' % value)

    def all(self):
        """Set this slice to all units between the miniumum and maximum"""
        self.vfrom = self.slice.min
        self.vto = self.slice.max

    def render(self, resolve=False):
        """Render the ranged value for a cronjob"""
        value = '*'
        if int(self.vfrom) > self.slice.min or int(self.vto) < self.slice.max:
            if self.vfrom == self.vto:
                value = unicode(self.vfrom)
            else:
                value = _render_values([self.vfrom, self.vto], '-', resolve)
        if self.seq != 1:
            value += "/%d" % self.seq
        if value != '*' and SYSTEMV:
            value = ','.join([unicode(val) for val in self.range()])
        return value

    def range(self):
        """Returns the range of this cron slice as a iterable list"""
        return range(int(self.vfrom), int(self.vto) + 1, self.seq)

    def every(self, value):
        """Set the sequence value for this range."""
        self.seq = int(value)

    def __lt__(self, value):
        return int(self.vfrom) < int(value)

    def __gt__(self, value):
        return int(self.vto) > int(value)

    def __int__(self):
        return int(self.vfrom)

    def __str__(self):
        return self.__unicode__()

    def __unicode__(self):
        return self.render()


class OrderedVariableList(OrderedDict):
    """An ordered dictionary with a linked list containing
    the previous OrderedVariableList which this list depends.

    Duplicates in this list are weeded out in favour of the previous
    list in the chain.

    This is all in aid of the ENV variables list which must exist one
    per job in the chain.
    """

    def __init__(self, *args, **kw):
        self.job = kw.pop('job', None)
        super(OrderedVariableList, self).__init__(*args, **kw)

    @property
    def previous(self):
        """Returns the previous env in the list of jobs in the cron"""
        if self.job is not None and self.job.cron is not None:
            index = self.job.cron.crons.index(self.job)
            if index == 0:
                return self.job.cron.env
            return self.job.cron[index - 1].env
        return None

    def all(self):
        """
        Returns the full dictionary, everything from this dictionary
        _Golden all those in the chain above us.
        """
        if self.job is not None:
            ret = self.previous.all().copy()
            ret.update(self)
            return ret
        return self.copy()

    def __getitem__(self, key):
        previous = self.previous
        if key in self:
            return super(OrderedVariableList, self).__getitem__(key)
        elif previous is not None:
            return previous.all()[key]
        raise KeyError("Environment Variable '%s' not found." % key)

    def __str__(self):
        """Constructs to variable list output used in cron jobs"""
        ret = []
        for key, value in self.items():
            if self.previous:
                if self.previous.all().get(key, None) == value:
                    continue
            if ' ' in unicode(value) or value == '':
                value = '"%s"' % value
            ret.append("%s=%s" % (key, unicode(value)))
        ret.append('')
        return "\n".join(ret)


if __name__ == '__main__':
    print(CronItem('xx', 'x'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : textin_types
# @Time         : 2025/3/23 11:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# BASE_URL = "https://api.textin.com"
BASE_URL = "https://api.textin.com/ai/service/v1"


# dict_to_model(d)

class DynamicModel(BaseModel):
    pdf_pwd: Optional[str] = 'None'
    dpi: int = '144'
    page_start: int = '0'
    page_count: int = '1000'
    apply_document_tree: int = '0'
    markdown_details: int = '1'
    page_details: int = '0'
    table_flavor: str = 'md'
    get_image: str = 'none'
    parse_mode: str = 'scan'


class WatermarkRemove(BaseModel):
    image: str  # path url base64 # from pydantic import HttpUrl
    response_format: Literal["url", "b64_json"] = 'b64_json'

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "image": "https://oss.ffire.cc/files/sese1.jpg",
                    "response_format": "url"

                },
            ]
        }


class PdfToMarkdown(BaseModel):
    data: str  # path url base64 # from pydantic import HttpUrl
    response_format: Literal["url", "b64_json"] = 'url'

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "data": "https://s3.ffire.cc/files/pdf_to_markdown.jpg",
                    "response_format": "url"

                },
            ]
        }

class CropEnhanceImage(BaseModel):
    data: str  # path url base64 # from pydantic import HttpUrl
    response_format: Literal["url", "b64_json"] = 'url'

    class Config:
        extra = "allow"

        json_schema_extra = {
            "examples": [
                {
                    "data": "https://s3.ffire.cc/files/pdf_to_markdown.jpg",
                    "response_format": "url"

                },
            ]
        }

class WatermarkRemoveResponse(BaseModel):
    """{'code': 200,
     'duration': 375,
     'has_watermark': 1,
     'message': 'success',
     'result': {'image': },'version': '0.2.1'}
    """
    pass

    class Config:
        extra = "allow"


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tts
# @Time         : 2023/11/3 15:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 增加固定时长的生成
# https://github.com/rany2/edge-tts/blob/master/examples/streaming_with_subtitles.py

import edge_tts

from meutils.pipe import *
from meutils.decorators.retry import retrying


class EdgeTTS(object):

    def __init__(self):
        pass

    def create(
            self,
            text: Union[str, List[str]],
            role: str = '云希',
            rate: float = 0,
            volume: float = 0,
            filename: Optional[None] = None,
            **kwargs
    ):
        """ todo：增加字幕合成，增加 多进程，多线程
            for i, srtitem in enumerate(tqdm(pysrt.open('subtitle.srt'))):
                create(srtitem.text, filename=f"wav/{i:0>6}.wav")
        """
        if isinstance(text, str):
            text = [text]

        acreate = partial(self.acreate, role=role, rate=rate, volume=volume, filename=filename)

        return text | xmap(acreate) | xAsyncio

    @alru_cache
    async def acreate(
            self,
            text: str,
            role: str = '云希',
            rate: float = 0,
            volume: float = 0,
            filename: Optional[None] = None,
    ):
        voices = (await edge_tts.VoicesManager.create()).find(**{'ShortName': role}) or [{}]  # todo: 优化
        voice = voices[0].get('ShortName', 'zh-CN-YunxiNeural')

        rate = f"{'+' if rate >= 0 else ''}{rate}%"
        volume = f"{'+' if volume >= 0 else ''}{volume}%"

        communicate = edge_tts.Communicate(text, voice=voice, rate=rate, volume=volume)  # todo: 文件流

        filename = filename or f"{time.time()}.mp3"  # f"{str(datetime.datetime.now())[:19]}.mp3"
        await communicate.save(filename)

        return filename

    @retrying()
    async def stream_acreate(
            self,
            text: str,
            voice: str = '云希',  # "晓晓", "晓伊", "云健", "云希", "云夏", "云扬", "辽宁晓北", "陕西晓妮"
            rate: float = 0,
            volume: float = 0,
            **kwargs
    ):
        voices = {
            '晓晓': 'zh-CN-XiaoxiaoNeural', '女声': 'zh-CN-XiaoxiaoNeural',
            '晓伊': 'zh-CN-XiaoyiNeural',
            '云健': 'zh-CN-YunjianNeural',
            '云希': 'zh-CN-YunxiNeural', '男声': 'zh-CN-YunxiNeural',
            '云夏': 'zh-CN-YunxiaNeural',
            '云扬': 'zh-CN-YunyangNeural',
            '辽宁晓北': 'zh-CN-liaoning-XiaobeiNeural',
            '陕西晓妮': 'zh-CN-shaanxi-XiaoniNeural',
        }
        voice = voices.get(voice, 'zh-CN-YunxiNeural')

        rate = f"{'+' if rate >= 0 else ''}{rate}%"
        volume = f"{'+' if volume >= 0 else ''}{volume}%"

        communicate = edge_tts.Communicate(text, voice=voice, rate=rate, volume=volume)
        async for chunk in communicate.stream():  # 构建生成器，流式输出， todo: 字幕结构体
            # {'type': 'WordBoundary', 'offset': 1000000, 'duration': 6750000, 'text': '你好'} # 断句字幕
            # https://github.com/rany2/edge-tts/blob/master/examples/streaming_with_subtitles.py
            # if chunk.get('type') == 'WordBoundary':
            #     logger.debug(chunk)
            #     continue
            if chunk.get('type') == "audio":
                yield chunk["data"]

    @staticmethod
    @lru_cache
    def find_voices(**kwargs: Any):  # @alru_cache
        """https://speech.platform.bing.com/consumer/speech/synthesize/readaloud/voices/list?trustedclienttoken=6A5AA1D4EAFF4E9FB37E23D68491D6F4
            find_voices(**{"Locale": "en-US"})
            find_voices(**{'ShortName': 'zh-CN-XiaoxiaoNeural'})
        """
        return asyncio.run(edge_tts.VoicesManager.create()).find(**kwargs or {"Locale": "zh-CN"})

    async def acreate_for_openai(
            self,
            input: str,
            model: str = 'edge-tts',
            voice: str = '云希',  # 男声 女声
            **kwargs
    ):
        data = {
            "text": input,
            "voice": voice,
        }
        return self.stream_acreate(**data)


if __name__ == '__main__':
    # print(EdgeTTS().create(['不知道'] * 10))

    cls = EdgeTTS()
    # cls.create('不知道')
    # cls.create('不知道')

    from meutils.async_utils import async2sync_generator

    input = "健身需要注意适度和平衡，过度的锻炼可能会导致身体受伤。因此，进行健身活动前，最好先咨询医生或专业的健身教练，制定一个适合自己的健身计划。一般来说，一周内进行150分钟的适度强度的有氧运动，或者75分钟的高强度有氧运动，加上每周两天的肌肉锻炼，就能达到保持健康的目标。"

    text = """
    陕西省，简称“陕”或“秦”，中华人民共和国省级行政区，省会西安，位于中国内陆腹地，黄河中游，东邻山西、河南，西连宁夏、甘肃，南抵四川、重庆、湖北，北接内蒙古，介于东经105°29′—111°15′，北纬31°42′—39°35′之间，总面积205624.3平方千米。 [1] [5]截至2022年11月，陕西省下辖10个地级市（其中省会西安为副省级市）、31个市辖区、7个县级市、69个县。 [121]截至2022年末，陕西省常住人口3956万人。
    """

    print(cls.find_voices(Locale="zh-CN"))
    for i in async2sync_generator(cls.stream_acreate(input)):
        print(i)

    # EDGE_TTS_DICT = {
    #     "用英语": "en-US-AriaNeural",
    #     "用日语": "ja-JP-NanamiNeural",
    #     "用法语": "fr-BE-CharlineNeural",
    #     "用韩语": "ko-KR-SunHiNeural",
    #     "用德语": "de-AT-JonasNeural",
    #     # add more here
    # }

    # arun(cls.acreate(input))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : token
# @Time         : 2024/7/19 13:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 优化扣费逻辑
import asyncio

from meutils.pipe import *
from meutils.schemas.oneapi import BASE_URL


@alru_cache(ttl=30)
async def get_api_key_money(api_key):
    headers = {
        'authorization': f'Bearer {api_key}'
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        tasks = [
            client.get("/v1/dashboard/billing/usage"),
            client.get("/v1/dashboard/billing/subscription")  # 查令牌
        ]
        response1, response2 = await asyncio.gather(*tasks)
        logger.debug(response1.status_code)
        logger.debug(response1.json())
        logger.debug(response2.status_code)
        logger.debug(response2.json())
        # {'error': {'message': '该令牌状态不可用 (request id: 20241029105800334407026WGK1hBhC)','type': 'new_api_error'}}

        # {"object":"list","total_usage":13665.746200000001}
        # {"object":"billing_subscription","has_payment_method":true,"soft_limit_usd":2000,"hard_limit_usd":2000,"system_hard_limit_usd":2000,"access_until":0}

        response1.raise_for_status()
        response2.raise_for_status()

        total_usage = response1.json()['total_usage'] / 100
        hard_limit_usd = response2.json()['hard_limit_usd']
        return hard_limit_usd - total_usage


if __name__ == '__main__':
    arun(get_api_key_money(os.getenv("OPENAI_API_KEY_GUOCHAN")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : headers
# @Time         : 2025/2/23 00:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from fastapi import FastAPI, Request, Depends, HTTPException
from typing import Dict, Optional


def get_headers(request: Request):
    dic = dict(request.headers)
    _dic = {k.replace('-', '_'): v for k, v in dic.items()}

    return {**dic, **_dic}


if __name__ == '__main__':
    def get_headers():
        d = {"upstream-base-url": 'xx'}
        d = {}
        dic = {}
        # upstream_base_url = headers.get('upstream-base-url')
        if d:
            dic = {k.replace('-', '_'): v for k, v in d.items()}

        return {**d, **dic}


    print(get_headers())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/4/10 16:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from meutils.llm.clients import AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params
from meutils.schemas.image_types import ImageRequest, ImagesResponse


class Images(object):
    def __init__(
            self,
            base_url: Optional[str] = None,
            api_key: Optional[str] = None,
            http_client: Optional[httpx.AsyncClient] = None
    ):
        self.base_url = base_url
        self.api_key = api_key

        self.client = AsyncOpenAI(base_url=self.base_url, api_key=self.api_key, http_client=http_client)

    async def generate(self, request: ImageRequest):
        data = to_openai_params(request)
        response = await self.client.images.generate(**data)  # todo: 兼容 response_format="b64_json"

        return response
